@@ -78,7 +78,9 @@ static my_bool  verbose=0,tFlag=0,cFlag=0,dFlag=0,quick=0, extended_insert = 0,
   opt_alldbs=0,opt_create_db=0,opt_first_slave=0,
   opt_autocommit=0,opt_master_data,opt_disable_keys=0,opt_xml=0,
   opt_delete_master_logs=0, tty_password=0,
-  opt_single_transaction=0, opt_comments= 0;
+  opt_single_transaction=0, opt_comments= 0,
+  opt_hex_blob;
+static ulong opt_max_allowed_packet, opt_net_buffer_length;
 static MYSQL  mysql_connection,*sock=0;
 static char  insert_pat[12 * 1024],*opt_password=0,*current_user=0,
              *current_host=0,*path=0,*fields_terminated=0,
@@ -87,7 +89,6 @@ static char  insert_pat[12 * 1024],*opt_password=0,*current_user=0,
 static uint     opt_mysql_port=0;
 static my_string opt_mysql_unix_port=0;
 static int   first_error=0;
-extern ulong net_buffer_length;
 static DYNAMIC_STRING extended_row;
 #include <sslopt-vars.h>
 FILE  *md_result_file;
@@ -113,6 +114,9 @@ static struct my_option my_long_options[] =
   {"character-sets-dir", OPT_CHARSETS_DIR,
    "Directory where character sets are", (gptr*) &charsets_dir,
    (gptr*) &charsets_dir, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"comments", 'i', "Write additional information.",
+   (gptr*) &opt_comments, (gptr*) &opt_comments, 0, GET_BOOL, NO_ARG,
+   1, 0, 0, 0, 0, 0},
   {"complete-insert", 'c', "Use complete insert statements.", (gptr*) &cFlag,
    (gptr*) &cFlag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"compress", 'C', "Use compression in server/client protocol.",
@@ -150,7 +154,7 @@ static struct my_option my_long_options[] =
    "Fields in the i.file are opt. enclosed by ...", (gptr*) &opt_enclosed,
    (gptr*) &opt_enclosed, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0 ,0, 0},
   {"fields-escaped-by", OPT_ESC, "Fields in the i.file are escaped by ...",
-   (gptr*) &escaped, (gptr*) &escaped, 0, GET_STR, NO_ARG, 0, 0, 0, 0, 0, 0},
+   (gptr*) &escaped, (gptr*) &escaped, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"first-slave", 'x', "Locks all tables across all databases.",
    (gptr*) &opt_first_slave, (gptr*) &opt_first_slave, 0, GET_BOOL, NO_ARG,
    0, 0, 0, 0, 0, 0},
@@ -165,6 +169,9 @@ static struct my_option my_long_options[] =
    0, 0, 0, 0, 0, 0},
   {"help", '?', "Display this help message and exit.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
+  {"hex-blob", OPT_HEXBLOB, "Dump binary strings (CHAR BINARY, "
+    "VARCHAR BINARY, BLOB) in hexadecimal format.",
+   (gptr*) &opt_hex_blob, (gptr*) &opt_hex_blob, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"host", 'h', "Connect to host.", (gptr*) &current_host,
    (gptr*) &current_host, 0, GET_STR_ALLOC, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"lines-terminated-by", OPT_LTB, "Lines in the i.file are terminated by ...",
@@ -179,10 +186,6 @@ static struct my_option my_long_options[] =
    "Wrap tables with autocommit/commit statements.",
    (gptr*) &opt_autocommit, (gptr*) &opt_autocommit, 0, GET_BOOL, NO_ARG,
    0, 0, 0, 0, 0, 0},
-  {"single-transaction", OPT_TRANSACTION,
-   "Dump all tables in single transaction to get consistent snapshot. Mutually exclusive with --lock-tables.",
-   (gptr*) &opt_single_transaction, (gptr*) &opt_single_transaction, 0, GET_BOOL, NO_ARG,
-   0, 0, 0, 0, 0, 0},
   {"no-create-db", 'n',
    "'CREATE DATABASE /*!32312 IF NOT EXISTS*/ db_name;' will not be put in the output. The above line will be added otherwise, if --databases or --all-databases option was given.}",
    (gptr*) &opt_create_db, (gptr*) &opt_create_db, 0, GET_BOOL, NO_ARG, 0, 0,
@@ -191,9 +194,6 @@ static struct my_option my_long_options[] =
    (gptr*) &tFlag, (gptr*) &tFlag, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"no-data", 'd', "No row information.", (gptr*) &dFlag, (gptr*) &dFlag, 0,
    GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-  {"set-variable", 'O',
-   "Change the value of a variable. Please note that this option is deprecated; you can set variables directly with --variable-name=value.",
-   0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"opt", OPT_OPTIMIZE,
    "Same as --add-drop-table --add-locks --all --quick --extended-insert --lock-tables --disable-keys",
    0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0},
@@ -215,6 +215,13 @@ static struct my_option my_long_options[] =
   {"result-file", 'r',
    "Direct output to a given file. This option should be used in MSDOS, because it prevents new line '\\n' from being converted to '\\r\\n' (carriage return + line feed).",
    0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"set-variable", 'O',
+   "Change the value of a variable. Please note that this option is deprecated; you can set variables directly with --variable-name=value.",
+   0, 0, 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+  {"single-transaction", OPT_TRANSACTION,
+   "Dump all tables in single transaction to get consistent snapshot. Mutually exclusive with --lock-tables.",
+   (gptr*) &opt_single_transaction, (gptr*) &opt_single_transaction, 0, GET_BOOL, NO_ARG,
+   0, 0, 0, 0, 0, 0},
   {"socket", 'S', "Socket file to use for connection.",
    (gptr*) &opt_mysql_unix_port, (gptr*) &opt_mysql_unix_port, 0, GET_STR,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
@@ -238,16 +245,13 @@ static struct my_option my_long_options[] =
   {"xml", 'X', "Dump a database as well formed XML.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
   {"max_allowed_packet", OPT_MAX_ALLOWED_PACKET, "",
-    (gptr*) &max_allowed_packet, (gptr*) &max_allowed_packet, 0,
+    (gptr*) &opt_max_allowed_packet, (gptr*) &opt_max_allowed_packet, 0,
     GET_ULONG, REQUIRED_ARG, 24*1024*1024, 4096, 
    (longlong) 2L*1024L*1024L*1024L, MALLOC_OVERHEAD, 1024, 0},
   {"net_buffer_length", OPT_NET_BUFFER_LENGTH, "",
-    (gptr*) &net_buffer_length, (gptr*) &net_buffer_length, 0,
+    (gptr*) &opt_net_buffer_length, (gptr*) &opt_net_buffer_length, 0,
     GET_ULONG, REQUIRED_ARG, 1024*1024L-1025, 4096, 16*1024L*1024L,
     MALLOC_OVERHEAD-1024, 1024, 0},
-  {"comments", 'i', "Write additional information.",
-   (gptr*) &opt_comments, (gptr*) &opt_comments, 0, GET_BOOL, NO_ARG,
-   1, 0, 0, 0, 0, 0},
   {0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
@@ -267,11 +271,13 @@ static char *quote_name(const char *name, char *buff, my_bool force);
 static void print_quoted_xml(FILE *output, char *fname, char *str, uint len);
 static const char *check_if_ignore_table(const char *table_name);
 
+#include <help_start.h>
 
 static void print_version(void)
 {
   printf("%s  Ver %s Distrib %s, for %s (%s)\n",my_progname,DUMP_VERSION,
-   MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
+         MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
+  NETWARE_SET_SCREEN_MODE(1);
 } /* print_version */
 
 
@@ -281,8 +287,10 @@ static void short_usage_sub(void)
   printf("OR     %s [OPTIONS] --databases [OPTIONS] DB1 [DB2 DB3...]\n",
 	 my_progname);
   printf("OR     %s [OPTIONS] --all-databases [OPTIONS]\n", my_progname);
+  NETWARE_SET_SCREEN_MODE(1);
 }
 
+
 static void usage(void)
 {
   print_version();
@@ -302,6 +310,8 @@ static void short_usage(void)
   printf("For more options, use %s --help\n", my_progname);
 }
 
+#include <help_end.h>
+
 
 static void write_header(FILE *sql_file, char *db_name)
 {
@@ -399,6 +409,10 @@ get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
 static int get_options(int *argc, char ***argv)
 {
   int ho_error;
+  MYSQL_PARAMETERS *mysql_params= mysql_get_parameters();
+
+  opt_max_allowed_packet= *mysql_params->p_max_allowed_packet;
+  opt_net_buffer_length= *mysql_params->p_net_buffer_length;
 
   md_result_file= stdout;
   load_defaults("my",load_default_groups,argc,argv);
@@ -406,6 +420,9 @@ static int get_options(int *argc, char ***argv)
   if ((ho_error=handle_options(argc, argv, my_long_options, get_one_option)))
     exit(ho_error);
 
+  *mysql_params->p_max_allowed_packet= opt_max_allowed_packet;
+  *mysql_params->p_net_buffer_length= opt_net_buffer_length;
+
   if (opt_delayed)
     opt_lock=0;				/* Can't have lock with delayed */
   if (!path && (enclosed || opt_enclosed || escaped || lines_terminated ||
@@ -1067,7 +1084,7 @@ static void dumpTable(uint numFields, char *table)
     if (opt_lock)
       fprintf(md_result_file,"LOCK TABLES %s WRITE;\n", opt_quoted_table);
 
-    total_length=net_buffer_length;		/* Force row break */
+    total_length= opt_net_buffer_length;		/* Force row break */
     row_break=0;
     rownr=0;
     init_length=(uint) strlen(insert_pat)+4;
@@ -1091,6 +1108,7 @@ static void dumpTable(uint numFields, char *table)
 
       for (i = 0; i < mysql_num_fields(res); i++)
       {
+        int is_blob;
 	if (!(field = mysql_fetch_field(res)))
 	{
 	  sprintf(query,"%s: Not enough fields from table %s! Aborting.\n",
@@ -1099,6 +1117,14 @@ static void dumpTable(uint numFields, char *table)
 	  error= EX_CONSCHECK;
 	  goto err;
 	}
+	
+        is_blob= (opt_hex_blob && (field->flags & BINARY_FLAG) &&
+                  (field->type == FIELD_TYPE_STRING ||
+                   field->type == FIELD_TYPE_VAR_STRING ||
+                   field->type == FIELD_TYPE_BLOB ||
+                   field->type == FIELD_TYPE_LONG_BLOB ||
+                   field->type == FIELD_TYPE_MEDIUM_BLOB ||
+                   field->type == FIELD_TYPE_TINY_BLOB)) ? 1 : 0;
 	if (extended_insert)
 	{
 	  ulong length = lengths[i];
@@ -1113,18 +1139,37 @@ static void dumpTable(uint numFields, char *table)
 	    {
 	      if (!IS_NUM_FIELD(field))
 	      {
+	        /*
+	          "length * 2 + 2" is OK for both HEX and non-HEX modes:
+	          - In HEX mode we need exactly 2 bytes per character
+	          plus 2 bytes for '0x' prefix.
+	          - In non-HEX mode we need up to 2 bytes per character,
+	          plus 2 bytes for leading and trailing '\'' characters.
+	        */
 		if (dynstr_realloc(&extended_row,length * 2+2))
 		{
 		  fputs("Aborting dump (out of memory)",stderr);
 		  error= EX_EOM;
 		  goto err;
 		}
-		dynstr_append(&extended_row,"\'");
-		extended_row.length +=
+                if (opt_hex_blob && is_blob)
+                {
+                  dynstr_append(&extended_row, "0x");
+                  extended_row.length+= mysql_hex_string(extended_row.str + 
+                                                         extended_row.length,
+                                                         row[i], length);
+                  extended_row.str[extended_row.length]= '\0';
+                }
+                else
+                {
+		  dynstr_append(&extended_row,"\'");
+		  extended_row.length +=
 		  mysql_real_escape_string(&mysql_connection,
-					   &extended_row.str[extended_row.length],row[i],length);
-		extended_row.str[extended_row.length]='\0';
-		dynstr_append(&extended_row,"\'");
+		  			   &extended_row.str[extended_row.length],
+		  			   row[i],length);
+		  extended_row.str[extended_row.length]='\0';
+		  dynstr_append(&extended_row,"\'");
+		}
 	      }
 	      else
 	      {
@@ -1167,7 +1212,19 @@ static void dumpTable(uint numFields, char *table)
 	      if (opt_xml)
 		print_quoted_xml(md_result_file, field->name, row[i],
 				 lengths[i]);
-	      else
+	      else if (opt_hex_blob && is_blob)
+              { /* sakaik got this idea. */
+                ulong counter;
+                char xx[4];
+                unsigned char *ptr= row[i];
+                fputs("0x", md_result_file);
+                for (counter = 0; counter < lengths[i]; counter++)
+                {
+                  sprintf(xx, "%02X", ptr[counter]);
+                  fputs(xx, md_result_file);
+                }
+              }
+              else
 		unescape(md_result_file, row[i], lengths[i]);
 	    }
 	    else
@@ -1212,7 +1269,7 @@ static void dumpTable(uint numFields, char *table)
 	ulong row_length;
 	dynstr_append(&extended_row,")");
         row_length = 2 + extended_row.length;
-        if (total_length + row_length < net_buffer_length)
+        if (total_length + row_length < opt_net_buffer_length)
         {
 	  total_length += row_length;
 	  fputc(',',md_result_file);		/* Always row break */
