@@ -46,7 +46,7 @@ static char	*opt_password=0, *current_user=0,
 		*escaped=0, *opt_columns=0, *default_charset;
 static uint     opt_mysql_port=0;
 static my_string opt_mysql_unix_port=0;
-static my_string opt_ignore_lines=0;
+static longlong opt_ignore_lines= -1;
 #include <sslopt-vars.h>
 
 static struct my_option my_long_options[] =
@@ -90,7 +90,7 @@ static struct my_option my_long_options[] =
   {"ignore", 'i', "If duplicate unique key was found, keep old row.",
    (gptr*) &ignore, (gptr*) &ignore, 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
   {"ignore-lines", OPT_IGN_LINES, "Ignore first n lines of data infile.",
-   (gptr*) &opt_ignore_lines, (gptr*) &opt_ignore_lines, 0, GET_STR,
+   (gptr*) &opt_ignore_lines, (gptr*) &opt_ignore_lines, 0, GET_LL,
    REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
   {"lines-terminated-by", OPT_LTB, "Lines in the i.file are terminated by ...",
    (gptr*) &lines_terminated, (gptr*) &lines_terminated, 0, GET_STR,
@@ -134,14 +134,16 @@ static struct my_option my_long_options[] =
 
 static const char *load_default_groups[]= { "mysqlimport","client",0 };
 
+#include <help_start.h>
+
 static void print_version(void)
 {
   printf("%s  Ver %s Distrib %s, for %s (%s)\n" ,my_progname,
 	  IMPORT_VERSION, MYSQL_SERVER_VERSION,SYSTEM_TYPE,MACHINE_TYPE);
+  NETWARE_SET_SCREEN_MODE(1);
 }
 
 
-
 static void usage(void)
 {
   print_version();
@@ -160,6 +162,7 @@ file. The SQL command 'LOAD DATA INFILE' is used to import the rows.\n");
   my_print_variables(my_long_options);
 }
 
+#include <help_end.h>
 
 static my_bool
 get_one_option(int optid, const struct my_option *opt __attribute__((unused)),
@@ -243,10 +246,8 @@ static int write_to_table(char *filename, MYSQL *sock)
   DBUG_ENTER("write_to_table");
   DBUG_PRINT("enter",("filename: %s",filename));
 
-  local_file= sock->unix_socket == 0 || opt_local_file;
-
   fn_format(tablename, filename, "", "", 1 | 2); /* removes path & ext. */
-  if (local_file)
+  if (! opt_local_file)
     strmov(hard_path,filename);
   else
     my_load_path(hard_path, filename, NULL); /* filename includes the path */
@@ -265,7 +266,7 @@ static int write_to_table(char *filename, MYSQL *sock)
   to_unix_path(hard_path);
   if (verbose)
   {
-    if (local_file)
+    if (opt_local_file)
       fprintf(stdout, "Loading data from LOCAL file: %s into %s\n",
 	      hard_path, tablename);
     else
@@ -274,7 +275,7 @@ static int write_to_table(char *filename, MYSQL *sock)
   }
   sprintf(sql_statement, "LOAD DATA %s %s INFILE '%s'",
 	  opt_low_priority ? "LOW_PRIORITY" : "",
-	  local_file ? "LOCAL" : "", hard_path);
+	  opt_local_file ? "LOCAL" : "", hard_path);
   end= strend(sql_statement);
   if (replace)
     end= strmov(end, " REPLACE");
@@ -290,8 +291,9 @@ static int write_to_table(char *filename, MYSQL *sock)
 		       " OPTIONALLY ENCLOSED BY");
   end= add_load_option(end, escaped, " ESCAPED BY");
   end= add_load_option(end, lines_terminated, " LINES TERMINATED BY");
-  if (opt_ignore_lines)
-    end= strmov(strmov(strmov(end, " IGNORE "), opt_ignore_lines), " LINES");
+  if (opt_ignore_lines >= 0)
+    end= strmov(longlong10_to_str(opt_ignore_lines, 
+				  strmov(end, " IGNORE "),10), " LINES");
   if (opt_columns)
     end= strmov(strmov(strmov(end, " ("), opt_columns), ")");
   *end= '\0';
