@@ -59,7 +59,7 @@
 #include <sys/stat.h>
 #include <violite.h>
 
-#define MAX_QUERY	65536
+#define MAX_QUERY     131072
 #define MAX_COLUMNS	256
 #define PAD_SIZE	128
 #define MAX_CONS	128
@@ -178,6 +178,17 @@ typedef struct
   int alloced;
 } VAR;
 
+#ifdef __NETWARE__
+/*
+  Netware doesn't proved environment variable substitution that is done
+  by the shell in unix environments. We do this in the following function:
+*/
+
+static char *subst_env_var(const char *cmd);
+static FILE *my_popen(const char *cmd, const char *mode);
+#define popen(A,B) my_popen((A),(B))
+#endif /* __NETWARE__ */
+
 VAR var_reg[10];
 /*Perl/shell-like variable registers */
 HASH var_hash;
@@ -890,7 +901,7 @@ int do_exec(struct st_query* q)
 
     while (fgets(buf, sizeof(buf), res_file))
       replace_dynstr_append_mem(ds, buf, strlen(buf));
-  
+
     if (glob_replace)
       free_replace();
 
@@ -913,6 +924,7 @@ int do_exec(struct st_query* q)
   DBUG_RETURN(error);
 }
 
+
 int var_query_set(VAR* v, const char* p, const char** p_end)
 {
   char* end = (char*)((p_end && *p_end) ? *p_end : p + strlen(p));
@@ -1081,7 +1093,7 @@ int do_sync_with_master2(const char* p)
   MYSQL_ROW row;
   MYSQL* mysql = &cur_con->mysql;
   char query_buf[FN_REFLEN+128];
-  int offset = 0;
+  int offset= 0, tries= 0;
   int rpl_parse;
 
   if (!master_pos.file[0])
@@ -1096,6 +1108,9 @@ int do_sync_with_master2(const char* p)
 
   sprintf(query_buf, "select master_pos_wait('%s', %ld)", master_pos.file,
 	  master_pos.pos + offset);
+
+wait_for_position:
+
   if (mysql_query(mysql, query_buf))
     die("line %u: failed in %s: %d: %s", start_lineno, query_buf,
 	mysql_errno(mysql), mysql_error(mysql));
@@ -1106,8 +1121,20 @@ int do_sync_with_master2(const char* p)
   if (!(row = mysql_fetch_row(res)))
     die("line %u: empty result in %s", start_lineno, query_buf);
   if (!row[0])
-    die("line %u: could not sync with master ('%s' returned NULL)", 
-        start_lineno, query_buf);
+  {
+    /*
+      It may be that the slave SQL thread has not started yet, though START
+      SLAVE has been issued ?
+    */
+    if (tries++ == 3)
+    {
+      die("line %u: could not sync with master ('%s' returned NULL)", 
+          start_lineno, query_buf);
+    }
+    sleep(1); /* So at most we will wait 3 seconds and make 4 tries */
+    mysql_free_result(res);
+    goto wait_for_position;
+  }
   mysql_free_result(res);
   last_result=0;
   if (rpl_parse)
@@ -1790,13 +1817,6 @@ int read_query(struct st_query** q_ptr)
   q->record_file[0] = 0;
   q->require_file=0;
   q->first_word_len = 0;
-  memcpy((gptr) q->expected_errno, (gptr) global_expected_errno,
-	 sizeof(global_expected_errno));
-  q->expected_errors=global_expected_errors;
-  q->abort_on_error = global_expected_errno[0] == 0;
-  bzero((gptr) global_expected_errno,sizeof(global_expected_errno));
-  global_expected_errors=0;
-
   q->type = Q_UNKNOWN;
   q->query_buf=q->query=0;
   if (read_line(read_query_buf, sizeof(read_query_buf)))
@@ -1805,8 +1825,16 @@ int read_query(struct st_query** q_ptr)
   if (*p == '#')
   {
     q->type = Q_COMMENT;
+    /* This goto is to avoid losing the "expected error" info. */
+    goto end;
   }
-  else if (p[0] == '-' && p[1] == '-')
+  memcpy((gptr) q->expected_errno, (gptr) global_expected_errno,
+	 sizeof(global_expected_errno));
+  q->expected_errors=global_expected_errors;
+  q->abort_on_error = global_expected_errno[0] == 0;
+  bzero((gptr) global_expected_errno,sizeof(global_expected_errno));
+  global_expected_errors=0;
+  if (p[0] == '-' && p[1] == '-')
   {
     q->type = Q_COMMENT_WITH_COMMAND;
     p+=2;					/* To calculate first word */
@@ -1841,6 +1869,8 @@ int read_query(struct st_query** q_ptr)
       *p1 = 0;
     }
   }
+
+end:
   while (*p && my_isspace(charset_info,*p))
     p++;
   if (!(q->query_buf=q->query=my_strdup(p,MYF(MY_WME))))
@@ -3458,3 +3488,102 @@ static void get_replace_column(struct st_query *q)
   }
   my_free(start, MYF(0));
 }
+
+#ifdef __NETWARE__
+
+/*
+  Substitute environment variables with text.
+
+  SYNOPSIS
+    subst_env_var()
+    arg			String that should be substitute
+
+  DESCRIPTION
+    This function takes a string as an input and replaces the
+    environment variables, that starts with '$' character, with it value.
+
+  NOTES
+    Return string must be freed with my_free()
+
+  RETURN
+    String with environment variables replaced.
+*/
+
+static char *subst_env_var(const char *str)
+{
+  char *result;
+  char *pos;
+
+  result= pos= my_malloc(MAX_QUERY, MYF(MY_FAE));
+  while (*str)
+  {
+    /*
+      need this only when we want to provide the functionality of
+      escaping through \ 'backslash'
+      if ((result == pos && *str=='$') ||
+          (result != pos && *str=='$' && str[-1] !='\\'))
+    */
+    if (*str == '$')
+    {
+      char env_var[256], *env_pos= env_var, *subst;
+
+      /* Search for end of environment variable */
+      for (str++;
+           *str && !isspace(*str) && *str != '\\' && *str != '/' &&
+             *str != '$';
+           str++)
+        *env_pos++= *str;
+      *env_pos= 0;
+
+      if (!(subst= getenv(env_var)))
+      {
+        my_free(result, MYF(0));
+        die("MYSQLTEST.NLM: Environment variable %s is not defined\n",
+            env_var);
+      }
+
+      /* get the string to be substitued for env_var  */
+      pos= strmov(pos, subst);
+      /* Process delimiter in *str again */
+    }
+    else
+      *pos++= *str++;
+  }
+  *pos= 0;
+  return result;
+}
+
+
+/*
+  popen replacement for Netware
+
+  SYNPOSIS
+    my_popen()
+    name		Command to execute (with possible env variables)
+    mode		Mode for popen.
+
+  NOTES
+    Environment variable expansion does not take place for popen function
+    on NetWare, so we use this function to wrap around popen to do this.
+
+    For the moment we ignore 'mode' and always use 'r0'
+
+  RETURN
+    # >= 0	File handle
+    -1		Error
+*/
+
+#undef popen                                    /* Remove wrapper */
+
+FILE *my_popen(const char *cmd, const char *mode __attribute__((unused)))
+{
+  char *subst_cmd;
+  FILE *res_file;
+
+  subst_cmd= subst_env_var(cmd);
+  res_file= popen(subst_cmd, "r0");
+  my_free(subst_cmd, MYF(0));
+  return res_file;
+}
+
+#endif /* __NETWARE__ */
