@@ -5,7 +5,7 @@ The B-tree
 
 Created 6/2/1994 Heikki Tuuri
 *******************************************************/
-
+ 
 #include "btr0btr.h"
 
 #ifdef UNIV_NONINL
@@ -22,6 +22,25 @@ Created 6/2/1994 Heikki Tuuri
 #include "ibuf0ibuf.h"
 
 /*
+Latching strategy of the InnoDB B-tree
+--------------------------------------
+A tree latch protects all non-leaf nodes of the tree. Each node of a tree
+also has a latch of its own.
+
+A B-tree operation normally first acquires an S-latch on the tree. It
+searches down the tree and releases the tree latch when it has the
+leaf node latch. To save CPU time we do not acquire any latch on
+non-leaf nodes of the tree during a search, those pages are only bufferfixed.
+
+If an operation needs to restructure the tree, it acquires an X-latch on
+the tree before searching to a leaf node. If it needs, for example, to
+split a leaf,
+(1) InnoDB decides the split point in the leaf,
+(2) allocates a new page,
+(3) inserts the appropriate node pointer to the first non-leaf level,
+(4) releases the tree X-latch,
+(5) and then moves records from the leaf to the new allocated page.
+
 Node pointers
 -------------
 Leaf pages of a B-tree contain the index records stored in the
@@ -57,9 +76,6 @@ make them consecutive on disk if possible. From the other file segment
 we allocate pages for the non-leaf levels of the tree.
 */
 
-/* If this many inserts occur sequentially, it affects page split */
-#define BTR_PAGE_SEQ_INSERT_LIMIT	5
-
 /******************************************************************
 Creates a new index page to the tree (not the root, and also not
 used in page reorganization). */
@@ -127,9 +143,6 @@ btr_root_get(
 	ulint	space;
 	ulint	root_page_no;
 	page_t*	root;
-
-	ut_ad(mtr_memo_contains(mtr, dict_tree_get_lock(tree), MTR_MEMO_X_LOCK)
-	  || mtr_memo_contains(mtr, dict_tree_get_lock(tree), MTR_MEMO_S_LOCK));
 	
 	space = dict_tree_get_space(tree);
 	root_page_no = dict_tree_get_page(tree);
@@ -255,6 +268,7 @@ btr_page_create(
 	ut_ad(mtr_memo_contains(mtr, buf_block_align(page),
 			      				MTR_MEMO_PAGE_X_FIX));
 	page_create(page, mtr);
+	buf_block_align(page)->check_index_page_at_flush = TRUE;
 	
 	btr_page_set_index_id(page, tree->id, mtr);
 }
@@ -282,7 +296,9 @@ btr_page_alloc_for_ibuf(
 
 	new_page = buf_page_get(dict_tree_get_space(tree), node_addr.page,
 							RW_X_LATCH, mtr);
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(new_page, SYNC_TREE_NODE_NEW);
+#endif /* UNIV_SYNC_DEBUG */
 
 	flst_remove(root + PAGE_HEADER + PAGE_BTR_IBUF_FREE_LIST,
 		    new_page + PAGE_HEADER + PAGE_BTR_IBUF_FREE_LIST_NODE,
@@ -314,8 +330,6 @@ btr_page_alloc(
 	page_t*		new_page;
 	ulint		new_page_no;
 
-	ut_ad(mtr_memo_contains(mtr, dict_tree_get_lock(tree),
-							MTR_MEMO_X_LOCK));
 	if (tree->type & DICT_IBUF) {
 
 		return(btr_page_alloc_for_ibuf(tree, mtr));
@@ -342,7 +356,9 @@ btr_page_alloc(
 
 	new_page = buf_page_get(dict_tree_get_space(tree), new_page_no,
 							RW_X_LATCH, mtr);
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(new_page, SYNC_TREE_NODE_NEW);
+#endif /* UNIV_SYNC_DEBUG */
 							
 	return(new_page);
 }	
@@ -383,7 +399,7 @@ btr_get_size(
 		
 		n += fseg_n_reserved_pages(seg_header, &dummy, &mtr);		
 	} else {
-		ut_a(0);
+		ut_error;
 	}
 
 	mtr_commit(&mtr);
@@ -572,28 +588,29 @@ btr_page_get_father_for_rec(
 
 	if (btr_node_ptr_get_child_page_no(node_ptr) !=
                                                 buf_frame_get_page_no(page)) {
-		fprintf(stderr,
-"InnoDB: Dump of the child page:\n");
+		fputs("InnoDB: Dump of the child page:\n", stderr);
 		buf_page_print(buf_frame_align(page));
-		fprintf(stderr,
-"InnoDB: Dump of the parent page:\n");
+		fputs("InnoDB: Dump of the parent page:\n", stderr);
 		buf_page_print(buf_frame_align(node_ptr));
 
-      		fprintf(stderr,
-"InnoDB: Corruption of an index tree: table %s, index %s,\n"
+		fputs("InnoDB: Corruption of an index tree: table ", stderr);
+		ut_print_name(stderr,
+			UT_LIST_GET_FIRST(tree->tree_indexes)->table_name);
+		fputs(", index ", stderr);
+		ut_print_name(stderr,
+			UT_LIST_GET_FIRST(tree->tree_indexes)->name);
+		fprintf(stderr, ",\n"
 "InnoDB: father ptr page no %lu, child page no %lu\n",
-                    (UT_LIST_GET_FIRST(tree->tree_indexes))->table_name,
-                    (UT_LIST_GET_FIRST(tree->tree_indexes))->name,
-                    btr_node_ptr_get_child_page_no(node_ptr),
-                    buf_frame_get_page_no(page));
+			btr_node_ptr_get_child_page_no(node_ptr),
+			buf_frame_get_page_no(page));
      		page_rec_print(page_rec_get_next(page_get_infimum_rec(page)));
      		page_rec_print(node_ptr);
 
-      		fprintf(stderr,
+		fputs(
 "InnoDB: You should dump + drop + reimport the table to fix the\n"
 "InnoDB: corruption. If the crash happens at the database startup, see\n"
-"InnoDB: section 6.1 of http://www.innodb.com/ibman.html about forcing\n"
-"InnoDB: recovery. Then dump + drop + reimport.\n");
+"InnoDB: http://dev.mysql.com/doc/mysql/en/Forcing_recovery.html about\n"
+"InnoDB: forcing recovery. Then dump + drop + reimport.\n", stderr);
 	}
 
 	ut_a(btr_node_ptr_get_child_page_no(node_ptr) ==
@@ -648,8 +665,9 @@ btr_create(
 		ibuf_hdr_frame = fseg_create(space, 0,
 				IBUF_HEADER + IBUF_TREE_SEG_HEADER, mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 		buf_page_dbg_add_level(ibuf_hdr_frame, SYNC_TREE_NODE_NEW);
-
+#endif /* UNIV_SYNC_DEBUG */
 		ut_ad(buf_frame_get_page_no(ibuf_hdr_frame)
  						== IBUF_HEADER_PAGE_NO);
 		/* Allocate then the next page to the segment: it will be the
@@ -674,7 +692,9 @@ btr_create(
 
 	page_no = buf_frame_get_page_no(frame);
 	
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(frame, SYNC_TREE_NODE_NEW);
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (type & DICT_IBUF) {
 		/* It is an insert buffer tree: initialize the free list */
@@ -689,11 +709,14 @@ btr_create(
 									mtr);
 		/* The fseg create acquires a second latch on the page,
 		therefore we must declare it: */
+#ifdef UNIV_SYNC_DEBUG
 		buf_page_dbg_add_level(frame, SYNC_TREE_NODE_NEW);
+#endif /* UNIV_SYNC_DEBUG */
 	}
 	
 	/* Create a new index page on the the allocated segment page */
 	page = page_create(frame, mtr);
+	buf_block_align(page)->check_index_page_at_flush = TRUE;
 
 	/* Set the index id of the page */
 	btr_page_set_index_id(page, index_id, mtr);
@@ -806,9 +829,16 @@ btr_page_reorganize_low(
 {
 	page_t*	new_page;
 	ulint	log_mode;
+	ulint	data_size1;
+	ulint	data_size2;
+	ulint	max_ins_size1;
+	ulint	max_ins_size2;
 
 	ut_ad(mtr_memo_contains(mtr, buf_block_align(page),
 			      				MTR_MEMO_PAGE_X_FIX));
+	data_size1 = page_get_data_size(page);
+	max_ins_size1 = page_get_max_insert_size_after_reorganize(page, 1);
+
 	/* Write the log record */
 	mlog_write_initial_log_record(page, MLOG_PAGE_REORGANIZE, mtr);
 
@@ -828,6 +858,7 @@ btr_page_reorganize_low(
 	segment headers, next page-field, etc.) is preserved intact */
 
 	page_create(page, mtr);
+	buf_block_align(page)->check_index_page_at_flush = TRUE;
 	
 	/* Copy the records from the temporary space to the recreated page;
 	do not copy the lock bits yet */
@@ -842,6 +873,19 @@ btr_page_reorganize_low(
 		lock_move_reorganize_page(page, new_page);
 	}
 
+	data_size2 = page_get_data_size(page);
+	max_ins_size2 = page_get_max_insert_size_after_reorganize(page, 1);
+
+	if (data_size1 != data_size2 || max_ins_size1 != max_ins_size2) {
+		buf_page_print(page);
+		buf_page_print(new_page);
+	        fprintf(stderr,
+"InnoDB: Error: page old data size %lu new data size %lu\n"
+"InnoDB: Error: page old max ins size %lu new max ins size %lu\n"
+"InnoDB: Submit a detailed bug report to http://bugs.mysql.com\n",
+			data_size1, data_size2, max_ins_size1, max_ins_size2);
+	}
+
 	buf_frame_free(new_page);
 
 	/* Restore logging mode */
@@ -868,7 +912,7 @@ btr_parse_page_reorganize(
 /*======================*/
 			/* out: end of log record or NULL */
 	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
+	byte*	end_ptr __attribute__((unused)), /* in: buffer end */
 	page_t*	page,	/* in: page or NULL */
 	mtr_t*	mtr)	/* in: mtr or NULL */
 {
@@ -900,6 +944,7 @@ btr_page_empty(
 	segment headers, next page-field, etc.) is preserved intact */
 
 	page_create(page, mtr);
+	buf_block_align(page)->check_index_page_at_flush = TRUE;
 }
 
 /*****************************************************************
@@ -1006,7 +1051,7 @@ btr_root_raise_and_insert(
 
 	/* We play safe and reset the free bits for the new page */
 
-/*	printf("Root raise new page no %lu\n",
+/*	fprintf(stderr, "Root raise new page no %lu\n",
 					buf_frame_get_page_no(new_page)); */
 
 	ibuf_reset_free_bits(UT_LIST_GET_FIRST(tree->tree_indexes),
@@ -1039,18 +1084,18 @@ btr_page_get_split_rec_to_left(
 	page = btr_cur_get_page(cursor);
 	insert_point = btr_cur_get_rec(cursor);
 
-	if ((page_header_get_ptr(page, PAGE_LAST_INSERT)
-	    == page_rec_get_next(insert_point))
-	    && (page_header_get_field(page, PAGE_DIRECTION) == PAGE_LEFT)
-	    && ((page_header_get_field(page, PAGE_N_DIRECTION)
-	     	 			>= BTR_PAGE_SEQ_INSERT_LIMIT)
-	     	|| (page_header_get_field(page, PAGE_N_DIRECTION) + 1
-	     	 			>= page_get_n_recs(page)))) {
+	if (page_header_get_ptr(page, PAGE_LAST_INSERT)
+	    == page_rec_get_next(insert_point)) {
 
 	     	infimum = page_get_infimum_rec(page);
-	     	 			
-		if ((infimum != insert_point)
-		    && (page_rec_get_next(infimum) != insert_point)) {
+		
+		/* If the convergence is in the middle of a page, include also
+		the record immediately before the new insert to the upper
+		page. Otherwise, we could repeatedly move from page to page
+		lots of records smaller than the convergence point. */
+
+		if (infimum != insert_point
+		    && page_rec_get_next(infimum) != insert_point) {
 
 			*split_rec = insert_point;
 		} else {
@@ -1084,29 +1129,29 @@ btr_page_get_split_rec_to_right(
 	page = btr_cur_get_page(cursor);
 	insert_point = btr_cur_get_rec(cursor);
 
-	if ((page_header_get_ptr(page, PAGE_LAST_INSERT) == insert_point)
-	    && (page_header_get_field(page, PAGE_DIRECTION) == PAGE_RIGHT)
-	    && ((page_header_get_field(page, PAGE_N_DIRECTION)
-	     	 			>= BTR_PAGE_SEQ_INSERT_LIMIT)
-	     	|| (page_header_get_field(page, PAGE_N_DIRECTION) + 1
-	     	 			>= page_get_n_recs(page)))) {
+	/* We use eager heuristics: if the new insert would be right after
+	the previous insert on the same page, we assume that there is a
+	pattern of sequential inserts here. */
+
+	if (page_header_get_ptr(page, PAGE_LAST_INSERT) == insert_point) {
 
 	     	supremum = page_get_supremum_rec(page);
 	     	 			
-		if ((page_rec_get_next(insert_point) != supremum)
-		    && (page_rec_get_next(page_rec_get_next(insert_point))
-			!= supremum)
-		    && (page_rec_get_next(page_rec_get_next(
-					page_rec_get_next(insert_point)))
-			!= supremum)) {
+		if (page_rec_get_next(insert_point) != supremum
+		    && page_rec_get_next(page_rec_get_next(insert_point))
+			!= supremum) {
 
-			/* If there are >= 3 user records up from the insert
-			point, split all but 2 off */
-
-			*split_rec = page_rec_get_next(page_rec_get_next(
-					page_rec_get_next(insert_point)));
+			/* If there are >= 2 user records up from the insert
+			point, split all but 1 off. We want to keep one because
+			then sequential inserts can use the adaptive hash
+			index, as they can do the necessary checks of the right
+			search position just by looking at the records on this
+			page. */
+		
+			*split_rec = page_rec_get_next(
+					page_rec_get_next(insert_point));
 		} else {
-			/* Else split at inserted record */
+			/* Else split at the new record to insert */
 	     		*split_rec = NULL;
 	     	}
 
@@ -1469,7 +1514,7 @@ btr_page_split_and_insert(
 	page_t*		insert_page;
 	page_cur_t*	page_cursor;
 	rec_t*		first_rec;
-	byte*		buf;
+	byte*		buf = 0; /* remove warning */
 	rec_t*		move_limit;
 	ibool		insert_will_fit;
 	ulint		n_iterations = 0;
@@ -1479,7 +1524,9 @@ func_start:
 	
 	ut_ad(mtr_memo_contains(mtr, dict_tree_get_lock(tree),
 							MTR_MEMO_X_LOCK));
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(dict_tree_get_lock(tree), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	page = btr_cur_get_page(cursor);
 
@@ -1553,7 +1600,7 @@ func_start:
 
 	/* 5. Move then the records to the new page */
 	if (direction == FSP_DOWN) {
-/*		printf("Split left\n"); */
+/*		fputs("Split left\n", stderr); */
 
 		page_move_rec_list_start(new_page, page, move_limit, mtr);
 		left_page = new_page;
@@ -1561,7 +1608,7 @@ func_start:
 
 		lock_update_split_left(right_page, left_page);
 	} else {
-/*		printf("Split right\n"); */
+/*		fputs("Split right\n", stderr); */
 
 		page_move_rec_list_end(new_page, page, move_limit, mtr);
 		left_page = page;
@@ -1597,7 +1644,7 @@ func_start:
 		ibuf_update_free_bits_for_two_pages_low(cursor->index,
 							left_page,
 							right_page, mtr);
-		/* printf("Split and insert done %lu %lu\n",
+		/* fprintf(stderr, "Split and insert done %lu %lu\n",
 				buf_frame_get_page_no(left_page),
 				buf_frame_get_page_no(right_page)); */
 		return(rec);
@@ -1617,7 +1664,7 @@ func_start:
 		/* We play safe and reset the free bits for new_page */
 		ibuf_reset_free_bits(cursor->index, new_page);
 
-		/* printf("Split second round %lu\n",
+		/* fprintf(stderr, "Split second round %lu\n",
 					buf_frame_get_page_no(page)); */
 		n_iterations++;
 		ut_ad(n_iterations < 2);
@@ -1631,7 +1678,7 @@ func_start:
 
 	ibuf_update_free_bits_for_two_pages_low(cursor->index, left_page,
 							right_page, mtr);
-	/* printf("Split and insert done %lu %lu\n",
+	/* fprintf(stderr, "Split and insert done %lu %lu\n",
 				buf_frame_get_page_no(left_page),
 				buf_frame_get_page_no(right_page)); */
 
@@ -1647,7 +1694,7 @@ static
 void
 btr_level_list_remove(
 /*==================*/
-	dict_tree_t*	tree,	/* in: index tree */
+	dict_tree_t*	tree __attribute__((unused)), /* in: index tree */
 	page_t*		page,	/* in: page to remove */
 	mtr_t*		mtr)	/* in: mtr */
 {	
@@ -1878,7 +1925,7 @@ btr_compress(
 	left_page_no = btr_page_get_prev(page, mtr);
 	right_page_no = btr_page_get_next(page, mtr);
 
-/*	printf("Merge left page %lu right %lu \n", left_page_no,
+/*	fprintf(stderr, "Merge left page %lu right %lu \n", left_page_no,
 							right_page_no); */
 
 	node_ptr = btr_page_get_father_node_ptr(tree, page, mtr);
@@ -1927,11 +1974,20 @@ btr_compress(
 
 		btr_page_reorganize(merge_page, mtr);
 
+		max_ins_size = page_get_max_insert_size(merge_page, n_recs);
+
 		ut_ad(page_validate(merge_page, cursor->index));
 		ut_ad(page_get_max_insert_size(merge_page, n_recs)
 							== max_ins_size_reorg);
 	}
 
+	if (data_size > max_ins_size) {
+
+		/* Add fault tolerance, though this should never happen */
+
+		return;
+	}
+
 	btr_search_drop_page_hash_index(page);
 
 	/* Remove the page from the level list */
@@ -2119,8 +2175,9 @@ btr_print_size(
 	mtr_t		mtr;
 
 	if (tree->type & DICT_IBUF) {
-		printf(
-	"Sorry, cannot print info of an ibuf tree: use ibuf functions\n");
+		fputs(
+	"Sorry, cannot print info of an ibuf tree: use ibuf functions\n",
+			stderr);
 
 		return;
 	}
@@ -2131,14 +2188,14 @@ btr_print_size(
 
 	seg = root + PAGE_HEADER + PAGE_BTR_SEG_TOP;
 
-	printf("INFO OF THE NON-LEAF PAGE SEGMENT\n");
+	fputs("INFO OF THE NON-LEAF PAGE SEGMENT\n", stderr);
 	fseg_print(seg, &mtr);
 
 	if (!(tree->type & DICT_UNIVERSAL)) {
 
 		seg = root + PAGE_HEADER + PAGE_BTR_SEG_LEAF;
 
-		printf("INFO OF THE LEAF PAGE SEGMENT\n");
+		fputs("INFO OF THE LEAF PAGE SEGMENT\n", stderr);
 		fseg_print(seg, &mtr);
 	}
 
@@ -2166,7 +2223,7 @@ btr_print_recursive(
 	
 	ut_ad(mtr_memo_contains(mtr, buf_block_align(page),
 							MTR_MEMO_PAGE_X_FIX));
-	printf("NODE ON LEVEL %lu page number %lu\n",
+	fprintf(stderr, "NODE ON LEVEL %lu page number %lu\n",
 		btr_page_get_level(page, mtr), buf_frame_get_page_no(page));
 	
 	page_print(page, width, width);
@@ -2212,8 +2269,8 @@ btr_print_tree(
 	mtr_t	mtr;
 	page_t*	root;
 
-	printf("--------------------------\n");
-	printf("INDEX TREE PRINT\n");
+	fputs("--------------------------\n"
+		"INDEX TREE PRINT\n", stderr);
 
 	mtr_start(&mtr);
 
@@ -2270,48 +2327,100 @@ btr_check_node_ptr(
 }
 
 /****************************************************************
-Checks the size and number of fields in a record based on the definition of
-the index. */
+Display identification information for a record. */
 static
-ibool
-btr_index_rec_validate(
-/*====================*/
-				/* out: TRUE if ok */
+void
+btr_index_rec_validate_report(
+/*==========================*/
+	page_t*		page,	/* in: index page */
 	rec_t*		rec,	/* in: index record */
 	dict_index_t*	index)	/* in: index */
 {
-	dtype_t* type;
-	byte*	data;
+	fputs("InnoDB: Record in ", stderr);
+	dict_index_name_print(stderr, index);
+	fprintf(stderr, ", page %lu, at offset %lu\n",
+		buf_frame_get_page_no(page), (ulint)(rec - page));
+}
+
+/****************************************************************
+Checks the size and number of fields in a record based on the definition of
+the index. */
+
+ibool
+btr_index_rec_validate(
+/*====================*/
+					/* out: TRUE if ok */
+	rec_t*		rec,		/* in: index record */
+	dict_index_t*	index,		/* in: index */
+	ibool		dump_on_error)	/* in: TRUE if the function
+					should print hex dump of record
+					and page on error */
+{
 	ulint	len;
 	ulint	n;
 	ulint	i;
-	char	err_buf[1000];
+	page_t*	page;
+
+	page = buf_frame_align(rec);
 	
+	if (index->type & DICT_UNIVERSAL) {
+	        /* The insert buffer index tree can contain records from any
+	        other index: we cannot check the number of fields or
+	        their length */
+
+	        return(TRUE);
+	}
+
 	n = dict_index_get_n_fields(index);
 
 	if (rec_get_n_fields(rec) != n) {
-		fprintf(stderr, "Record has %lu fields, should have %lu\n",
-				rec_get_n_fields(rec), n);
+		btr_index_rec_validate_report(page, rec, index);
+		fprintf(stderr, "InnoDB: has %lu fields, should have %lu\n",
+			rec_get_n_fields(rec), n);
 
-		rec_sprintf(err_buf, 900, rec);
-	  	fprintf(stderr, "InnoDB: record %s\n", err_buf);
+		if (!dump_on_error) {
+
+			return(FALSE);
+		}
+
+		buf_page_print(page);
+
+		fputs("InnoDB: corrupt record ", stderr);
+		rec_print(stderr, rec);
+		putc('\n', stderr);
 
 		return(FALSE);
 	}
 
 	for (i = 0; i < n; i++) {
-		data = rec_get_nth_field(rec, i, &len);
+		dtype_t*	type = dict_index_get_nth_type(index, i);
 
-		type = dict_index_get_nth_type(index, i);
+		rec_get_nth_field(rec, i, &len);
 		
-		if (len != UNIV_SQL_NULL && dtype_is_fixed_size(type)
-		    && len != dtype_get_fixed_size(type)) {
+		if ((dict_index_get_nth_field(index, i)->prefix_len == 0
+		    && len != UNIV_SQL_NULL && dtype_is_fixed_size(type)
+		    && len != dtype_get_fixed_size(type))
+		   ||
+		   (dict_index_get_nth_field(index, i)->prefix_len > 0
+		    && len != UNIV_SQL_NULL && dtype_is_fixed_size(type)
+		    && len !=
+			   dict_index_get_nth_field(index, i)->prefix_len)) {
+
+			btr_index_rec_validate_report(page, rec, index);
 			fprintf(stderr,
-			"Record field %lu len is %lu, should be %lu\n",
+"InnoDB: field %lu len is %lu, should be %lu\n",
 				i, len, dtype_get_fixed_size(type));
 
-			rec_sprintf(err_buf, 900, rec);
-	  		fprintf(stderr, "InnoDB: record %s\n", err_buf);
+			if (!dump_on_error) {
+	
+				return(FALSE);
+			}
+
+			buf_page_print(page);
+
+			fputs("InnoDB: corrupt record ", stderr);
+			rec_print(stderr, rec);
+			putc('\n', stderr);
 
 			return(FALSE);
 		}
@@ -2331,7 +2440,6 @@ btr_index_page_validate(
 	page_t*		page,	/* in: index page */
 	dict_index_t*	index)	/* in: index */
 {
-	rec_t*		rec;
 	page_cur_t 	cur;
 	ibool		ret	= TRUE;
 	
@@ -2339,15 +2447,14 @@ btr_index_page_validate(
 	page_cur_move_to_next(&cur);
 
 	for (;;) {
-		rec = (&cur)->rec;
-
 		if (page_cur_is_after_last(&cur)) {
+
 			break;
 		}
 
-		if (!btr_index_rec_validate(rec, index)) {
+		if (!btr_index_rec_validate(cur.rec, index, TRUE)) {
 
-			ret = FALSE;
+			return(FALSE);
 		}
 
 		page_cur_move_to_next(&cur);
@@ -2356,6 +2463,46 @@ btr_index_page_validate(
 	return(ret);	
 }
 
+/****************************************************************
+Report an error on one page of an index tree. */
+static
+void
+btr_validate_report1(
+				/* out: TRUE if ok */
+	dict_index_t*	index,	/* in: index */
+	ulint		level,	/* in: B-tree level */
+	page_t*		page)	/* in: index page */
+{
+	fprintf(stderr, "InnoDB: Error in page %lu of ",
+		buf_frame_get_page_no(page));
+	dict_index_name_print(stderr, index);
+	if (level) {
+		fprintf(stderr, ", index tree level %lu", level);
+	}
+	putc('\n', stderr);
+}
+
+/****************************************************************
+Report an error on two pages of an index tree. */
+static
+void
+btr_validate_report2(
+				/* out: TRUE if ok */
+	dict_index_t*	index,	/* in: index */
+	ulint		level,	/* in: B-tree level */
+	page_t*		page1,	/* in: first index page */
+	page_t*		page2)	/* in: second index page */
+{
+	fprintf(stderr, "InnoDB: Error in pages %lu and %lu of ",
+		buf_frame_get_page_no(page1),
+		buf_frame_get_page_no(page2));
+	dict_index_name_print(stderr, index);
+	if (level) {
+		fprintf(stderr, ", index tree level %lu", level);
+	}
+	putc('\n', stderr);
+}
+
 /****************************************************************
 Validates index tree level. */
 static
@@ -2368,7 +2515,7 @@ btr_validate_level(
 {
 	ulint		space;
 	page_t*		page;
-	page_t*		right_page;
+	page_t*		right_page = 0; /* remove warning */
 	page_t*		father_page;
 	page_t*		right_father_page;
 	rec_t*		node_ptr;
@@ -2381,7 +2528,6 @@ btr_validate_level(
 	ibool		ret	= TRUE;
 	dict_index_t*	index;
 	mtr_t		mtr;
-	char		err_buf[1000];
 	
 	mtr_start(&mtr);
 
@@ -2404,25 +2550,23 @@ btr_validate_level(
 
 	index = UT_LIST_GET_FIRST(tree->tree_indexes);
 	
-	/* Now we are on the desired level */
+	/* Now we are on the desired level. Loop through the pages on that
+	level. */
 loop:
 	mtr_x_lock(dict_tree_get_lock(tree), &mtr);
 
 	/* Check ordering etc. of records */
 
 	if (!page_validate(page, index)) {
-		fprintf(stderr, "Error in page %lu in index %s\n",
-			buf_frame_get_page_no(page), index->name);
+		btr_validate_report1(index, level, page);
 
 		ret = FALSE;
-	}
+	} else if (level == 0) {
+		/* We are on level 0. Check that the records have the right
+		number of fields, and field lengths are right. */
 
-	if (level == 0) {
 		if (!btr_index_page_validate(page, index)) {
- 			fprintf(stderr,
-				"Error in page %lu in index %s, level %lu\n",
-				buf_frame_get_page_no(page), index->name,
-								level);
+
 			ret = FALSE;
 		}
 	}
@@ -2444,22 +2588,22 @@ loop:
 			page_rec_get_next(page_get_infimum_rec(right_page)),
 			UT_LIST_GET_FIRST(tree->tree_indexes)) >= 0) {
 
- 			fprintf(stderr,
-			"InnoDB: Error on pages %lu and %lu in index %s\n",
-				buf_frame_get_page_no(page),
-				right_page_no,
-				index->name);
+			btr_validate_report2(index, level, page, right_page);
 
-			fprintf(stderr,
-			"InnoDB: records in wrong order on adjacent pages\n");
+			fputs("InnoDB: records in wrong order"
+				" on adjacent pages\n", stderr);
 
-			rec_sprintf(err_buf, 900,
-				page_rec_get_prev(page_get_supremum_rec(page)));
-	  		fprintf(stderr, "InnoDB: record %s\n", err_buf);
+			buf_page_print(page);
+			buf_page_print(right_page);
 
-			rec_sprintf(err_buf, 900,
-			page_rec_get_next(page_get_infimum_rec(right_page)));
-	  		fprintf(stderr, "InnoDB: record %s\n", err_buf);
+			fputs("InnoDB: record ", stderr);
+			rec_print(stderr, page_rec_get_prev(
+				page_get_supremum_rec(page)));
+			putc('\n', stderr);
+			fputs("InnoDB: record ", stderr);
+			rec_print(stderr, page_rec_get_next(
+				page_get_infimum_rec(right_page)));
+			putc('\n', stderr);
 
 	  		ret = FALSE;
 	  	}
@@ -2475,42 +2619,39 @@ loop:
 		/* Check father node pointers */
 	
 		node_ptr = btr_page_get_father_node_ptr(tree, page, &mtr);
+		father_page = buf_frame_align(node_ptr);
 
 		if (btr_node_ptr_get_child_page_no(node_ptr) !=
 						buf_frame_get_page_no(page)
 		   || node_ptr != btr_page_get_father_for_rec(tree, page,
 		   	page_rec_get_prev(page_get_supremum_rec(page)),
 								&mtr)) {
- 			fprintf(stderr,
-			"InnoDB: Error on page %lu in index %s\n",
-				buf_frame_get_page_no(page),
-				index->name);
+			btr_validate_report1(index, level, page);
 
-			fprintf(stderr,
-			"InnoDB: node pointer to the page is wrong\n");
+			fputs("InnoDB: node pointer to the page is wrong\n",
+				stderr);
 
-			rec_sprintf(err_buf, 900, node_ptr);
-				
-	  		fprintf(stderr, "InnoDB: node ptr %s\n", err_buf);
+			buf_page_print(father_page);
+			buf_page_print(page);
 
-			fprintf(stderr,
+			fputs("InnoDB: node ptr ", stderr);
+			rec_print(stderr, node_ptr);
+
+			fprintf(stderr, "\n"
 				"InnoDB: node ptr child page n:o %lu\n",
 				btr_node_ptr_get_child_page_no(node_ptr));
 
-			rec_sprintf(err_buf, 900,
+			fputs("InnoDB: record on page ", stderr);
+			rec_print(stderr,
 			 	btr_page_get_father_for_rec(tree, page,
 		   	 	page_rec_get_prev(page_get_supremum_rec(page)),
 					&mtr));
-
-	  		fprintf(stderr, "InnoDB: record on page %s\n",
-								err_buf);
+			putc('\n', stderr);
 		   	ret = FALSE;
 
 		   	goto node_ptr_fails;
 		}
 
-		father_page = buf_frame_align(node_ptr);
-
 		if (btr_page_get_level(page, &mtr) > 0) {
 			heap = mem_heap_create(256);
 		
@@ -2523,24 +2664,19 @@ loop:
 
 			if (cmp_dtuple_rec(node_ptr_tuple, node_ptr) != 0) {
 
-	 			fprintf(stderr,
-				  "InnoDB: Error on page %lu in index %s\n",
-					buf_frame_get_page_no(page),
-					index->name);
+				btr_validate_report1(index, level, page);
 
-	  			fprintf(stderr,
-                	"InnoDB: Error: node ptrs differ on levels > 0\n");
-							
-				rec_sprintf(err_buf, 900, node_ptr);
-				
-	  			fprintf(stderr, "InnoDB: node ptr %s\n",
-								err_buf);
-				rec_sprintf(err_buf, 900,
-				  page_rec_get_next(
+				buf_page_print(father_page);
+				buf_page_print(page);
+
+				fputs("InnoDB: Error: node ptrs differ"
+					" on levels > 0\n"
+					"InnoDB: node ptr ", stderr);
+				rec_print(stderr, node_ptr);
+				fputs("InnoDB: first rec ", stderr);
+				rec_print(stderr, page_rec_get_next(
 					page_get_infimum_rec(page)));
-				
-	  			fprintf(stderr, "InnoDB: first rec %s\n",
-								err_buf);
+				putc('\n', stderr);
 		   		ret = FALSE;
 				mem_heap_free(heap);
 
@@ -2572,13 +2708,16 @@ loop:
 				if (right_node_ptr !=
 						page_rec_get_next(node_ptr)) {
 					ret = FALSE;
-					fprintf(stderr,
-			"InnoDB: node pointer to the right page is wrong\n");
+					fputs(
+			"InnoDB: node pointer to the right page is wrong\n",
+					stderr);
 
-	 				fprintf(stderr,
-				  "InnoDB: Error on page %lu in index %s\n",
-					buf_frame_get_page_no(page),
-					index->name);
+					btr_validate_report1(index, level,
+						page);
+
+					buf_page_print(father_page);
+					buf_page_print(page);
+					buf_page_print(right_page);
 				}
 			} else {
 				right_father_page = buf_frame_align(
@@ -2588,26 +2727,34 @@ loop:
 					   		page_get_infimum_rec(
 							right_father_page))) {
 					ret = FALSE;
-					fprintf(stderr,
-			"InnoDB: node pointer 2 to the right page is wrong\n");
+					fputs(
+			"InnoDB: node pointer 2 to the right page is wrong\n",
+					stderr);
 
-	 				fprintf(stderr,
-				  "InnoDB: Error on page %lu in index %s\n",
-					buf_frame_get_page_no(page),
-					index->name);
+					btr_validate_report1(index, level,
+						page);
+
+					buf_page_print(father_page);
+					buf_page_print(right_father_page);
+					buf_page_print(page);
+					buf_page_print(right_page);
 				}
 
 				if (buf_frame_get_page_no(right_father_page)
 				   != btr_page_get_next(father_page, &mtr)) {
 
 					ret = FALSE;
-					fprintf(stderr,
-			"InnoDB: node pointer 3 to the right page is wrong\n");
+					fputs(
+			"InnoDB: node pointer 3 to the right page is wrong\n",
+					stderr);
 
-	 				fprintf(stderr,
-				  "InnoDB: Error on page %lu in index %s\n",
-					buf_frame_get_page_no(page),
-					index->name);
+					btr_validate_report1(index, level,
+						page);
+
+					buf_page_print(father_page);
+					buf_page_print(right_father_page);
+					buf_page_print(page);
+					buf_page_print(right_page);
 				}
 			}					
 		}
