@@ -19,7 +19,12 @@ Created 2/17/1996 Heikki Tuuri
 #include "btr0btr.h"
 #include "ha0ha.h"
 
+ulint	btr_search_this_is_zero = 0;	/* A dummy variable to fool the
+					compiler */
+
+#ifdef UNIV_SEARCH_PERF_STAT
 ulint	btr_search_n_succ	= 0;
+#endif /* UNIV_SEARCH_PERF_STAT */
 ulint	btr_search_n_hash_fail	= 0;
 
 byte	btr_sea_pad1[64];	/* padding to prevent other memory update
@@ -56,16 +61,20 @@ before hash index building is started */
 
 /************************************************************************
 Builds a hash index on a page with the given parameters. If the page already
-has a hash index with different parameters, the old hash index is removed. */
+has a hash index with different parameters, the old hash index is removed.
+If index is non-NULL, this function checks if n_fields and n_bytes are
+sensible values, and does not build a hash index if not. */
 static
 void
 btr_search_build_page_hash_index(
 /*=============================*/
-	page_t*	page,		/* in: index page, s- or x-latched */
-	ulint	n_fields,	/* in: hash this many full fields */
-	ulint	n_bytes,	/* in: hash this many bytes from the next
+	dict_index_t*	index,	/* in: index for which to build, or NULL if
+				not known */
+	page_t*		page,	/* in: index page, s- or x-latched */
+	ulint		n_fields,/* in: hash this many full fields */
+	ulint		n_bytes,/* in: hash this many bytes from the next
 				field */
-	ulint	side);		/* in: hash for searches from this side */
+	ulint		side);	/* in: hash for searches from this side */
 
 /*********************************************************************
 This function should be called before reserving any btr search mutex, if
@@ -86,8 +95,10 @@ btr_search_check_free_space_in_heap(void)
 	hash_table_t*	table;
 	mem_heap_t*	heap;
 
-	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED)
-			&& !rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED));
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	table = btr_search_sys->hash_index;
 
@@ -173,7 +184,9 @@ btr_search_info_create(
 }
 
 /*************************************************************************
-Updates the search info of an index about hash successes. */
+Updates the search info of an index about hash successes. NOTE that info
+is NOT protected by any semaphore, to save CPU time! Do not assume its fields
+are consistent. */
 static
 void
 btr_search_info_update_hash(
@@ -185,8 +198,10 @@ btr_search_info_update_hash(
 	ulint		n_unique;
 	int		cmp;
 
-	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED)
-			&& !rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED));
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	index = cursor->index;
 
@@ -295,7 +310,9 @@ set_new_recomm:
 }
 	
 /*************************************************************************
-Updates the block search info on hash successes. */
+Updates the block search info on hash successes. NOTE that info and
+block->n_hash_helps, n_fields, n_bytes, side are NOT protected by any
+semaphore, to save CPU time! Do not assume the fields are consistent. */
 static
 ibool
 btr_search_update_block_hash_info(
@@ -306,10 +323,12 @@ btr_search_update_block_hash_info(
 	buf_block_t*	block,	/* in: buffer block */
 	btr_cur_t*	cursor)	/* in: cursor */
 {
-	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED)
-			&& !rw_lock_own(&btr_search_latch, RW_LOCK_EX));
-	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_SHARED)
-				|| rw_lock_own(&(block->lock), RW_LOCK_EX));
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED));
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+	ut_ad(rw_lock_own(&((buf_block_t*) block)->lock, RW_LOCK_SHARED)
+		|| rw_lock_own(&((buf_block_t*) block)->lock, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(cursor);
 
 	info->last_hash_succ = FALSE;
@@ -387,9 +406,11 @@ btr_search_update_hash_ref(
 	dulint	tree_id;
 
 	ut_ad(cursor->flag == BTR_CUR_HASH_FAIL);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(&btr_search_latch, RW_LOCK_EX));
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_SHARED)
 				|| rw_lock_own(&(block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 	if (block->is_hashed
 	    && (info->n_hash_potential > 0)
 	    && (block->curr_n_fields == info->n_fields)
@@ -408,7 +429,9 @@ btr_search_update_hash_ref(
 		fold = rec_fold(rec, block->curr_n_fields,
 						block->curr_n_bytes, tree_id);
 
+#ifdef UNIV_SYNC_DEBUG
 		ut_ad(rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 		ha_insert_for_fold(btr_search_sys->hash_index, fold, rec);
 	}
@@ -425,12 +448,21 @@ btr_search_info_update_slow(
 {
 	buf_block_t*	block;
 	ibool		build_index;
-
-	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED)
-			&& !rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+	ulint*		params;
+	ulint*		params2;
+	
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED));
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	block = buf_block_align(btr_cur_get_rec(cursor));
 
+	/* NOTE that the following two function calls do NOT protect
+	info or block->n_fields etc. with any semaphore, to save CPU time!
+	We cannot assume the fields are consistent when we return from
+	those functions! */
+
 	btr_search_info_update_hash(info, cursor);
 
 	build_index = btr_search_update_block_hash_info(info, block, cursor);
@@ -439,7 +471,7 @@ btr_search_info_update_slow(
 
 		btr_search_check_free_space_in_heap();
 	}
-		
+	
 	if (cursor->flag == BTR_CUR_HASH_FAIL) {
 		/* Update the hash node reference, if appropriate */
 
@@ -453,10 +485,30 @@ btr_search_info_update_slow(
 	}
 
 	if (build_index) {
-		btr_search_build_page_hash_index(block->frame,
-						block->n_fields,
-						block->n_bytes,
-						block->side);
+		/* Note that since we did not protect block->n_fields etc.
+		with any semaphore, the values can be inconsistent. We have
+		to check inside the function call that they make sense. We
+		also malloc an array and store the values there to make sure
+		the compiler does not let the function call parameters change
+		inside the called function. It might be that the compiler
+		would optimize the call just to pass pointers to block. */
+
+		params = mem_alloc(3 * sizeof(ulint));
+		params[0] = block->n_fields;
+		params[1] = block->n_bytes;
+		params[2] = block->side;
+
+		/* Make sure the compiler cannot deduce the values and do
+		optimizations */
+
+		params2 = params + btr_search_this_is_zero;
+		
+		btr_search_build_page_hash_index(cursor->index,
+						block->frame,
+						params2[0],
+						params2[1],
+						params2[2]);
+		mem_free(params);
 	}
 }
 
@@ -471,6 +523,13 @@ btr_search_check_guess(
 				/* out: TRUE if success */
 	btr_cur_t*	cursor,	/* in: guessed cursor position */
 	ibool           can_only_compare_to_cursor_rec,
+	                        /* in: if we do not have a latch on the page
+				of cursor, but only a latch on
+			        btr_search_latch, then ONLY the columns
+				of the record UNDER the cursor are
+				protected, not the next or previous record
+				in the chain: we cannot look at the next or
+				previous record to check our guess! */
 	dtuple_t* 	tuple,	/* in: data tuple */
 	ulint		mode,	/* in: PAGE_CUR_L, PAGE_CUR_LE, PAGE_CUR_G,
 				or PAGE_CUR_GE */
@@ -530,7 +589,10 @@ btr_search_check_guess(
 	}
 
 	if (can_only_compare_to_cursor_rec) {
-	       return(FALSE);
+	        /* Since we could not determine if our guess is right just by
+	        looking at the record under the cursor, return FALSE */
+
+	        return(FALSE);
 	}
 
 	match = 0;
@@ -714,7 +776,9 @@ btr_search_guess_on_hash(
 
 		can_only_compare_to_cursor_rec = FALSE;
 
+#ifdef UNIV_SYNC_DEBUG
 		buf_page_dbg_add_level(page, SYNC_TREE_NODE_FROM_HASH);
+#endif /* UNIV_SYNC_DEBUG */
 	}
 
 	block = buf_block_align(page);
@@ -739,15 +803,21 @@ btr_search_guess_on_hash(
 
 		success = FALSE;
 /*
-		printf("Tree id %lu, page index id %lu fold %lu\n",
+		fprintf(stderr, "Tree id %lu, page index id %lu fold %lu\n",
 				ut_dulint_get_low(tree_id),
 				ut_dulint_get_low(btr_page_get_index_id(page)),
 				fold);
 */				
 	} else {
+	        /* If we only have the latch on btr_search_latch, not on the
+		page, it only protects the columns of the record the cursor
+		is positioned on. We cannot look at the next of the previous
+		record to determine if our guess for the cursor position is
+		right. */
+
 		success = btr_search_check_guess(cursor,
-					   can_only_compare_to_cursor_rec,
-						 tuple, mode, mtr);
+				               can_only_compare_to_cursor_rec,
+					       tuple, mode, mtr);
 	}
 	
 	if (!success) {
@@ -856,10 +926,12 @@ btr_search_drop_page_hash_index(
 	ulint		n_recs;
 	ulint*		folds;
 	ulint		i;
-	
-	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED)
-			&& !rw_lock_own(&btr_search_latch, RW_LOCK_EX));
-		
+
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_SHARED));
+	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
+
 	rw_lock_s_lock(&btr_search_latch);
 
 	block = buf_block_align(page);
@@ -873,9 +945,11 @@ btr_search_drop_page_hash_index(
 
 	table = btr_search_sys->hash_index;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_SHARED)
 	      			|| rw_lock_own(&(block->lock), RW_LOCK_EX)
 	      			|| (block->buf_fix_count == 0));
+#endif /* UNIV_SYNC_DEBUG */
 
 	n_fields = block->curr_n_fields;
 	n_bytes = block->curr_n_bytes;
@@ -971,12 +1045,14 @@ btr_search_drop_page_hash_when_freed(
 	/* We assume that if the caller has a latch on the page,
 	then the caller has already dropped the hash index for the page,
 	and we never get here. Therefore we can acquire the s-latch to
-	the page without fearing a deadlock. */
+	the page without having to fear a deadlock. */
 	
 	page = buf_page_get(space, page_no, RW_S_LATCH, &mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(page, SYNC_TREE_NODE_FROM_HASH);
-	
+#endif /* UNIV_SYNC_DEBUG */
+
 	btr_search_drop_page_hash_index(page);
 
 	mtr_commit(&mtr);
@@ -984,16 +1060,20 @@ btr_search_drop_page_hash_when_freed(
 
 /************************************************************************
 Builds a hash index on a page with the given parameters. If the page already
-has a hash index with different parameters, the old hash index is removed. */
+has a hash index with different parameters, the old hash index is removed.
+If index is non-NULL, this function checks if n_fields and n_bytes are
+sensible values, and does not build a hash index if not. */
 static
 void
 btr_search_build_page_hash_index(
 /*=============================*/
-	page_t*	page,		/* in: index page, s- or x-latched */
-	ulint	n_fields,	/* in: hash this many full fields */
-	ulint	n_bytes,	/* in: hash this many bytes from the next
+	dict_index_t*	index,	/* in: index for which to build, or NULL if
+				not known */
+	page_t*		page,	/* in: index page, s- or x-latched */
+	ulint		n_fields,/* in: hash this many full fields */
+	ulint		n_bytes,/* in: hash this many bytes from the next
 				field */
-	ulint	side)		/* in: hash for searches from this side */
+	ulint		side)	/* in: hash for searches from this side */
 {
 	hash_table_t*	table;
 	buf_block_t*	block;
@@ -1012,9 +1092,11 @@ btr_search_build_page_hash_index(
 	block = buf_block_align(page);
 	table = btr_search_sys->hash_index;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(!rw_lock_own(&btr_search_latch, RW_LOCK_EX));
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_SHARED)
 				|| rw_lock_own(&(block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	rw_lock_s_lock(&btr_search_latch);
 				
@@ -1036,9 +1118,17 @@ btr_search_build_page_hash_index(
 		return;
 	}
 
+	/* Check that the values for hash index build are sensible */
+	
 	if (n_fields + n_bytes == 0) {
 
-	        return;
+		return;
+	}
+
+	if (index && (dict_index_get_n_unique_in_tree(index) < n_fields
+		      || (dict_index_get_n_unique_in_tree(index) == n_fields
+		          && n_bytes > 0))) {
+		return;
 	}
 
 	/* Calculate and cache fold values and corresponding records into
@@ -1169,8 +1259,10 @@ btr_search_move_or_delete_hash_entries(
 	block = buf_block_align(page);
 	new_block = buf_block_align(new_page);
 
-	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_EX)
-			&& rw_lock_own(&(new_block->lock), RW_LOCK_EX));
+#ifdef UNIV_SYNC_DEBUG
+	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_EX));
+	ut_ad(rw_lock_own(&(new_block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	rw_lock_s_lock(&btr_search_latch);
 			
@@ -1197,8 +1289,8 @@ btr_search_move_or_delete_hash_entries(
 
 		ut_a(n_fields + n_bytes > 0);
 		
-		btr_search_build_page_hash_index(new_page, n_fields, n_bytes,
-									side);
+		btr_search_build_page_hash_index(NULL, new_page, n_fields,
+							n_bytes, side);
 		ut_a(n_fields == block->curr_n_fields);
 		ut_a(n_bytes == block->curr_n_bytes);
 		ut_a(side == block->curr_side);
@@ -1230,7 +1322,9 @@ btr_search_update_hash_on_delete(
 
 	block = buf_block_align(rec);
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (!block->is_hashed) {
 
@@ -1271,7 +1365,9 @@ btr_search_update_hash_node_on_insert(
 
 	block = buf_block_align(rec);
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (!block->is_hashed) {
 
@@ -1318,7 +1414,7 @@ btr_search_update_hash_on_insert(
 	dulint		tree_id;
 	ulint		fold;
 	ulint		ins_fold;
-	ulint		next_fold;
+	ulint		next_fold = 0; /* remove warning (??? bug ???) */
 	ulint		n_fields;
 	ulint		n_bytes;
 	ulint		side;
@@ -1332,7 +1428,9 @@ btr_search_update_hash_on_insert(
 
 	block = buf_block_align(rec);
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(rw_lock_own(&(block->lock), RW_LOCK_EX));
+#endif /* UNIV_SYNC_DEBUG */
 	
 	if (!block->is_hashed) {
 
@@ -1417,8 +1515,9 @@ check_next_rec:
 
 			ha_insert_for_fold(table, ins_fold, ins_rec);
 /*
-			printf("Hash insert for %s, fold %lu\n",
-					cursor->index->name, ins_fold);
+			fputs("Hash insert for ", stderr);
+			dict_index_name_print(stderr, cursor->index);
+			fprintf(stderr, " fold %lu\n", ins_fold);
 */
 		} else {
 			ha_insert_for_fold(table, next_fold, next_rec);
@@ -1431,75 +1530,6 @@ function_exit:
 	}
 }
 
-/************************************************************************
-Prints info of the search system. */
-
-void
-btr_search_print_info(void)
-/*=======================*/
-{
-	printf("SEARCH SYSTEM INFO\n");
-
-	rw_lock_x_lock(&btr_search_latch);
-
-/*	ha_print_info(btr_search_sys->hash_index); */
-
-	rw_lock_x_unlock(&btr_search_latch);
-}
-
-/************************************************************************
-Prints info of searches on an index. */
-
-void
-btr_search_index_print_info(
-/*========================*/
-	dict_index_t*	index)	/* in: index */
-{
-	btr_search_t*	info;
-
-	printf("INDEX SEARCH INFO\n");
-
-	rw_lock_x_lock(&btr_search_latch);
-
-	info = btr_search_get_info(index);
-
-	printf("Searches %lu, hash succ %lu, fail %lu, patt succ %lu\n",
-		info->n_searches, info->n_hash_succ, info->n_hash_fail,
-		info->n_patt_succ);
-
-	printf("Total of page cur short succ for all indexes %lu\n",
-							page_cur_short_succ);
-	rw_lock_x_unlock(&btr_search_latch);
-}
-
-/************************************************************************
-Prints info of searches on a table. */
-
-void
-btr_search_table_print_info(
-/*========================*/
-	char*	name)	/* in: table name */
-{
-	dict_table_t*	table;
-	dict_index_t*	index;
-
-	mutex_enter(&(dict_sys->mutex));
-
-	table = dict_table_get_low(name);
-
-	ut_a(table);
-	
-	mutex_exit(&(dict_sys->mutex));
-
-	index = dict_table_get_first_index(table);
-
-	while (index) {
-		btr_search_index_print_info(index);
-
-		index = dict_table_get_next_index(index);
-	}
-}
-
 /************************************************************************
 Validates the search system. */
 
@@ -1514,7 +1544,6 @@ btr_search_validate(void)
 	ulint		n_page_dumps	= 0;
 	ibool		ok		= TRUE;
 	ulint		i;
-	char		rec_str[500];
 	
 	rw_lock_x_lock(&btr_search_latch);
 
@@ -1535,9 +1564,9 @@ btr_search_validate(void)
 
 				fprintf(stderr,
 "  InnoDB: Error in an adaptive hash index pointer to page %lu\n"
-"ptr mem address %lu index id %lu %lu, node fold %lu, rec fold %lu\n",
-				buf_frame_get_page_no(page),
-				(ulint)(node->data),
+"ptr mem address %p index id %lu %lu, node fold %lu, rec fold %lu\n",
+					buf_frame_get_page_no(page),
+					node->data,
 			ut_dulint_get_high(btr_page_get_index_id(page)),
 			ut_dulint_get_low(btr_page_get_index_id(page)),
 			node->fold, rec_fold((rec_t*)(node->data),
@@ -1545,16 +1574,12 @@ btr_search_validate(void)
 					block->curr_n_bytes,
 					btr_page_get_index_id(page)));
 
-				rec_sprintf(rec_str, 450, (rec_t*)(node->data));
-
-	  			fprintf(stderr,
-					"InnoDB: Record %s\n"
-					"InnoDB: on that page.", rec_str);
-
-				fprintf(stderr,
-"Page mem address %lu, is hashed %lu, n fields %lu, n bytes %lu\n"
+				fputs("InnoDB: Record ", stderr);
+				rec_print(stderr, (rec_t*)(node->data));
+				fprintf(stderr, "\nInnoDB: on that page."
+"Page mem address %p, is hashed %lu, n fields %lu, n bytes %lu\n"
 "side %lu\n",
-			(ulint)page, block->is_hashed, block->curr_n_fields,
+			page, block->is_hashed, block->curr_n_fields,
 			block->curr_n_bytes, block->curr_side);
 
 				if (n_page_dumps < 20) {	
