@@ -133,6 +133,13 @@ Creates the seek mutexes used in positioned reads and writes. */
 void
 os_io_init_simple(void);
 /*===================*/
+/***************************************************************************
+Creates a temporary file. */
+
+FILE*
+os_file_create_tmpfile(void);
+/*========================*/
+			/* out: temporary file handle, or NULL on error */
 /********************************************************************
 A simple function to open or create a file. */
 
