@@ -212,7 +212,8 @@ os_file_get_last_error(void)
 		ut_print_timestamp(stderr);
 	     	fprintf(stderr,
   "  InnoDB: Operating system error number %lu in a file operation.\n"
-  "InnoDB: See http://www.innodb.com/ibman.php for installation help.\n",
+  "InnoDB: See http://dev.mysql.com/doc/mysql/en/InnoDB.html\n"
+  "InnoDB: for installation help.\n",
 		err);
 
 		if (err == ERROR_PATH_NOT_FOUND) {
@@ -227,8 +228,9 @@ os_file_get_last_error(void)
   "InnoDB: of the same name as a data file.\n"); 
 		} else {
 			 fprintf(stderr,
-  "InnoDB: See section 13.2 at http://www.innodb.com/ibman.php\n"
-  "InnoDB: about operating system error numbers.\n");
+  "InnoDB: Some operating system error numbers are described at\n"
+  "InnoDB: "
+  "http://dev.mysql.com/doc/mysql/en/Operating_System_error_codes.html\n");
 		}
 	}
 
@@ -251,7 +253,8 @@ os_file_get_last_error(void)
 
 	     	fprintf(stderr,
   "  InnoDB: Operating system error number %lu in a file operation.\n"
-  "InnoDB: See http://www.innodb.com/ibman.php for installation help.\n",
+  "InnoDB: See http://dev.mysql.com/doc/mysql/en/InnoDB.html\n"
+  "InnoDB: for installation help.\n",
 		err);
 
 		if (err == ENOENT) {
@@ -270,8 +273,9 @@ os_file_get_last_error(void)
 			 }
 
 			 fprintf(stderr,
-  "InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
-  "InnoDB: about operating system error numbers.\n");
+  "InnoDB: Some operating system error numbers are described at\n"
+  "InnoDB: "
+  "http://dev.mysql.com/doc/mysql/en/Operating_System_error_codes.html\n");
 		}
 	}
 
@@ -301,14 +305,11 @@ os_file_handle_error(
 /*=================*/
 				/* out: TRUE if we should retry the
 				operation */
-	os_file_t	file,	/* in: file pointer */
 	const char*	name,	/* in: name of a file or NULL */
 	const char*	operation)/* in: operation */
 {
 	ulint	err;
 
-	UT_NOT_USED(file);
-
 	err = os_file_get_last_error();
 	
 	if (err == OS_FILE_DISK_FULL) {
@@ -374,6 +375,75 @@ os_io_init_simple(void)
 	}
 }
 
+#ifndef UNIV_HOTBACKUP
+/*************************************************************************
+Creates a temporary file. This function is defined in ha_innodb.cc. */
+
+int
+innobase_mysql_tmpfile(void);
+/*========================*/
+			/* out: temporary file descriptor, or < 0 on error */
+#endif /* !UNIV_HOTBACKUP */
+
+/***************************************************************************
+Creates a temporary file. */
+
+FILE*
+os_file_create_tmpfile(void)
+/*========================*/
+			/* out: temporary file handle, or NULL on error */
+{
+	FILE*	file	= NULL;
+	int	fd	= -1;
+#ifdef UNIV_HOTBACKUP
+	int	tries;
+	for (tries = 10; tries--; ) {
+		char*	name = tempnam(fil_path_to_mysql_datadir, "ib");
+		if (!name) {
+			break;
+		}
+
+		fd = open(name,
+# ifdef __WIN__
+			O_SEQUENTIAL | O_SHORT_LIVED | O_TEMPORARY |
+# endif /* __WIN__ */
+			O_CREAT | O_EXCL | O_RDWR,
+			S_IREAD | S_IWRITE);
+		if (fd >= 0) {
+# ifndef __WIN__
+			unlink(name);
+# endif /* !__WIN__ */
+			free(name);
+			break;
+		}
+
+		ut_print_timestamp(stderr);
+		fprintf(stderr, "  InnoDB: Warning: "
+			"unable to create temporary file %s, retrying\n",
+			name);
+		free(name);
+	}
+#else /* UNIV_HOTBACKUP */
+	fd = innobase_mysql_tmpfile();
+#endif /* UNIV_HOTBACKUP */
+
+	if (fd >= 0) {
+		file = fdopen(fd, "w+b");
+	}
+
+	if (!file) {
+		ut_print_timestamp(stderr);
+		fprintf(stderr,
+			"  InnoDB: Error: unable to create temporary file;"
+			" errno: %d\n", errno);
+		if (fd >= 0) {
+			close(fd);
+		}
+	}
+
+	return(file);
+}
+
 /********************************************************************
 A simple function to open or create a file. */
 
@@ -430,7 +500,7 @@ try_again:
 	if (file == INVALID_HANDLE_VALUE) {
 		*success = FALSE;
 
-		retry = os_file_handle_error(file, name,
+		retry = os_file_handle_error(name,
 				create_mode == OS_FILE_OPEN ?
 				"open" : "create");
 		if (retry) {
@@ -472,7 +542,7 @@ try_again:
 	if (file == -1) {
 		*success = FALSE;
 
-		retry = os_file_handle_error(file, name,
+		retry = os_file_handle_error(name,
 				create_mode == OS_FILE_OPEN ?
 				"open" : "create");
 		if (retry) {
@@ -678,7 +748,7 @@ try_again:
 	if (file == INVALID_HANDLE_VALUE) {
 		*success = FALSE;
 
-		retry = os_file_handle_error(file, name,
+		retry = os_file_handle_error(name,
 				create_mode == OS_FILE_OPEN ?
 				"open" : "create");
 		if (retry) {
@@ -766,7 +836,7 @@ try_again:
 	if (file == -1) {
 		*success = FALSE;
 
-		retry = os_file_handle_error(file, name,
+		retry = os_file_handle_error(name,
 				create_mode == OS_FILE_OPEN ?
 				"open" : "create");
 		if (retry) {
@@ -801,7 +871,7 @@ os_file_close(
 		return(TRUE);
 	}
 
-	os_file_handle_error(file, NULL, "close");
+	os_file_handle_error(NULL, "close");
 	return(FALSE);
 #else
 	int	ret;
@@ -809,7 +879,7 @@ os_file_close(
 	ret = close(file);
 
 	if (ret == -1) {
-		os_file_handle_error(file, NULL, "close");
+		os_file_handle_error(NULL, "close");
 		return(FALSE);
 	}
 
@@ -1029,7 +1099,7 @@ os_file_flush(
 		return(TRUE);
 	}
 
-	os_file_handle_error(file, NULL, "flush");
+	os_file_handle_error(NULL, "flush");
 
 	/* It is a fatal error if a file flush does not succeed, because then
 	the database can get corrupt on disk */
@@ -1063,7 +1133,7 @@ os_file_flush(
 	fprintf(stderr,
 		"  InnoDB: Error: the OS said file flush did not succeed\n");
 
-	os_file_handle_error(file, NULL, "flush");
+	os_file_handle_error(NULL, "flush");
 
 	/* It is a fatal error if a file flush does not succeed, because then
 	the database can get corrupt on disk */
@@ -1111,12 +1181,12 @@ os_file_pread(
 
 	os_n_file_reads++;
 
-#ifdef HAVE_PREAD
+#if defined(HAVE_PREAD) && !defined(HAVE_BROKEN_PREAD)
         os_mutex_enter(os_file_count_mutex);
 	os_file_n_pending_preads++;
         os_mutex_exit(os_file_count_mutex);
 
-        n_bytes = pread(file, buf, n, offs);
+        n_bytes = pread(file, buf, (ssize_t)n, offs);
 
         os_mutex_enter(os_file_count_mutex);
 	os_file_n_pending_preads--;
@@ -1141,7 +1211,7 @@ os_file_pread(
 		return(ret);
 	}
 	
-	ret = read(file, buf, n);
+	ret = read(file, buf, (ssize_t)n);
 
 	os_mutex_exit(os_file_seek_mutexes[i]);
 
@@ -1186,12 +1256,12 @@ os_file_pwrite(
 
 	os_n_file_writes++;
 
-#ifdef HAVE_PWRITE
+#if defined(HAVE_PWRITE) && !defined(HAVE_BROKEN_PREAD)
         os_mutex_enter(os_file_count_mutex);
 	os_file_n_pending_pwrites++;
         os_mutex_exit(os_file_count_mutex);
 
-	ret = pwrite(file, buf, n, offs);
+	ret = pwrite(file, buf, (ssize_t)n, offs);
 
         os_mutex_enter(os_file_count_mutex);
 	os_file_n_pending_pwrites--;
@@ -1226,7 +1296,7 @@ os_file_pwrite(
 		return(ret);
 	}
 	
-	ret = write(file, buf, n);
+	ret = write(file, buf, (ssize_t)n);
 
 	if (srv_unix_file_flush_method != SRV_UNIX_LITTLESYNC
 	    && srv_unix_file_flush_method != SRV_UNIX_NOSYNC
@@ -1323,7 +1393,7 @@ try_again:
 #ifdef __WIN__
 error_handling:
 #endif
-	retry = os_file_handle_error(file, NULL, "read"); 
+	retry = os_file_handle_error(NULL, "read"); 
 
 	if (retry) {
 		goto try_again;
@@ -1399,8 +1469,9 @@ retry:
 		fprintf(stderr,
 "  InnoDB: Error: File pointer positioning to file %s failed at\n"
 "InnoDB: offset %lu %lu. Operating system error number %lu.\n"
-"InnoDB: Look from section 13.2 at http://www.innodb.com/ibman.php\n"
-"InnoDB: what the error number means.\n",
+"InnoDB: Some operating system error numbers are described at\n"
+"InnoDB: "
+"http://dev.mysql.com/doc/mysql/en/Operating_System_error_codes.html\n",
 			name, offset_high, offset,
 			(ulint)GetLastError());
 
@@ -1457,8 +1528,9 @@ retry:
 		}
 
 		fprintf(stderr,
-"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
-"InnoDB: about operating system error numbers.\n");
+"InnoDB: Some operating system error numbers are described at\n"
+"InnoDB: "
+"http://dev.mysql.com/doc/mysql/en/Operating_System_error_codes.html\n");
 
 		os_has_said_disk_full = TRUE;
 	}
@@ -1492,8 +1564,9 @@ retry:
 		}
 
 		fprintf(stderr,
-"InnoDB: See also section 13.2 at http://www.innodb.com/ibman.php\n"
-"InnoDB: about operating system error numbers.\n");
+"InnoDB: Some operating system error numbers are described at\n"
+"InnoDB: "
+"http://dev.mysql.com/doc/mysql/en/Operating_System_error_codes.html\n");
 
 		os_has_said_disk_full = TRUE;
 	}
@@ -2278,7 +2351,7 @@ try_again:
 
 	os_aio_array_free_slot(array, slot);
 
-	retry = os_file_handle_error(file, name,
+	retry = os_file_handle_error(name,
 			type == OS_FILE_READ ? "aio read" : "aio write");
 	if (retry) {
 
@@ -2378,7 +2451,7 @@ os_aio_windows_handle(
 		         ut_a(TRUE == os_file_flush(slot->file));
 		}
 	} else {
-		os_file_handle_error(slot->file, slot->name, "Windows aio");
+		os_file_handle_error(slot->name, "Windows aio");
 		
 		ret_val = FALSE;
 	}		  
@@ -2515,6 +2588,8 @@ restart:
 	/* NOTE! We only access constant fields in os_aio_array. Therefore
 	we do not have to acquire the protecting mutex yet */
 
+	srv_set_io_thread_op_info(global_segment,
+					"looking for i/o requests (a)");
 	ut_ad(os_aio_validate());
 	ut_ad(segment < array->n_segments);
 
@@ -2533,6 +2608,9 @@ restart:
 	
 	os_mutex_enter(array->mutex);
 
+	srv_set_io_thread_op_info(global_segment,
+					"looking for i/o requests (b)");
+
 	/* Check if there is a slot for which the i/o has already been
 	done */
 	
@@ -2645,6 +2723,8 @@ consecutive_loop:
 		}
 	}
 
+	srv_set_io_thread_op_info(global_segment, "consecutive i/o requests");
+
 	/* We have now collected n_consecutive i/o requests in the array;
 	allocate a single buffer which can hold all data, and perform the
 	i/o */
@@ -2720,6 +2800,9 @@ consecutive_loop:
 				    	ut_print_timestamp(stderr);
 				    	fprintf(stderr,
 "  InnoDB: ERROR: The page to be written seems corrupt!\n");
+				    	fprintf(stderr,
+"InnoDB: Writing a block of %lu bytes, currently writing at offset %lu\n",
+					(ulong)total_len, (ulong)len2);
 					buf_page_print(combined_buf + len2);
 				    	fprintf(stderr,
 "InnoDB: ERROR: The page to be written seems corrupt!\n");
@@ -2785,6 +2868,8 @@ slot_io_done:
 	return(ret);
 
 wait_for_io:
+	srv_set_io_thread_op_info(global_segment, "resetting wait event");
+
 	/* We wait here until there again can be i/os in the segment
 	of this thread */
 	
@@ -2876,9 +2961,17 @@ os_aio_print(
 	ulint		i;
 
 	for (i = 0; i < srv_n_file_io_threads; i++) {
-		fprintf(file, "I/O thread %lu state: %s (%s)\n", i,
+		fprintf(file, "I/O thread %lu state: %s (%s)", i,
 					srv_io_thread_op_info[i],
 					srv_io_thread_function[i]);
+
+#ifndef __WIN__
+        	if (os_aio_segment_wait_events[i]->is_set) {
+			fprintf(file, " ev set");
+		}
+#endif
+			
+		fprintf(file, "\n");
 	}
 
 	fputs("Pending normal aio reads:", file);
