@@ -19,9 +19,6 @@ Created 2/2/1994 Heikki Tuuri
 #include "btr0sea.h"
 #include "buf0buf.h"
 
-/* A cached template page used in page_create */
-page_t*	page_template	= NULL;
-
 /*			THE INDEX PAGE
 			==============
 		
@@ -78,7 +75,6 @@ page_dir_find_owner_slot(
 	page_t*			page;	
 	page_dir_slot_t*	slot;
 	rec_t*			original_rec	= rec;
-	char			err_buf[1000];
 	
 	ut_ad(page_rec_check(rec));
 
@@ -96,24 +92,25 @@ page_dir_find_owner_slot(
 
  		if (i == 0) {
 			fprintf(stderr,
-		"InnoDB: Probable data corruption on page %lu\n",
+			"InnoDB: Probable data corruption on page %lu\n"
+			"InnoDB: Original record ",
 			buf_frame_get_page_no(page));
 
-			rec_sprintf(err_buf, 900, original_rec);
+			rec_print(stderr, original_rec);
 
-	  		fprintf(stderr,
-		"InnoDB: Original record %s\n"
-		"InnoDB: on that page. Steps %lu.\n", err_buf, steps);
+			fprintf(stderr, "\n"
+			"InnoDB: on that page. Steps %lu.\n", steps);
 
-			rec_sprintf(err_buf, 900, rec);
-
-	  		fprintf(stderr,
-		"InnoDB: Cannot find the dir slot for record %s\n"
-		"InnoDB: on that page!\n", err_buf);
+			fputs(
+			"InnoDB: Cannot find the dir slot for record ",
+				stderr);
+			rec_print(stderr, rec);
+			fputs("\n"
+			"InnoDB: on that page!\n", stderr);
 
 			buf_page_print(page);
 
-	  		ut_a(0);
+	  		ut_error;
 	  	}
 
 		i--;
@@ -270,7 +267,7 @@ page_parse_create(
 /*==============*/
 			/* out: end of log record or NULL */
 	byte*	ptr,	/* in: buffer */
-	byte*	end_ptr,/* in: buffer end */
+	byte*	end_ptr __attribute__((unused)), /* in: buffer end */
 	page_t*	page,	/* in: page or NULL */
 	mtr_t*	mtr)	/* in: mtr or NULL */
 {
@@ -321,20 +318,6 @@ page_create(
 
 	fil_page_set_type(page, FIL_PAGE_INDEX);
 
-	/* If we have a page template, copy the page structure from there */
-
-	if (page_template) {
-		ut_memcpy(page + PAGE_HEADER,
-			  page_template + PAGE_HEADER, PAGE_HEADER_PRIV_END);
-		ut_memcpy(page + PAGE_DATA,
-			  page_template + PAGE_DATA,
-			  PAGE_SUPREMUM_END - PAGE_DATA);
-		ut_memcpy(page + UNIV_PAGE_SIZE - PAGE_EMPTY_DIR_START,
-			page_template + UNIV_PAGE_SIZE - PAGE_EMPTY_DIR_START,
-		  		PAGE_EMPTY_DIR_START - PAGE_DIR);
-		return(frame);
-	}
-	
 	heap = mem_heap_create(200);
 		
 	/* 3. CREATE THE INFIMUM AND SUPREMUM RECORDS */
@@ -343,7 +326,7 @@ page_create(
 	tuple = dtuple_create(heap, 1);
 	field = dtuple_get_nth_field(tuple, 0);
 
-	dfield_set_data(field, "infimum", strlen("infimum") + 1);
+	dfield_set_data(field,(char *) "infimum", strlen("infimum") + 1);
 	dtype_set(dfield_get_type(field), DATA_VARCHAR, DATA_ENGLISH, 20, 0);
 	
 	/* Set the corresponding physical record to its place in the page
@@ -353,7 +336,7 @@ page_create(
 	
 	infimum_rec = rec_convert_dtuple_to_rec(heap_top, tuple);
 
-	ut_ad(infimum_rec == page + PAGE_INFIMUM);
+	ut_a(infimum_rec == page + PAGE_INFIMUM);
 	
 	rec_set_n_owned(infimum_rec, 1);
 	rec_set_heap_no(infimum_rec, 0);
@@ -365,12 +348,12 @@ page_create(
 	tuple = dtuple_create(heap, 1);
 	field = dtuple_get_nth_field(tuple, 0);
 
-	dfield_set_data(field, "supremum", strlen("supremum") + 1);
+	dfield_set_data(field, (char *) "supremum", strlen("supremum") + 1);
 	dtype_set(dfield_get_type(field), DATA_VARCHAR, DATA_ENGLISH, 20, 0);
 
 	supremum_rec = rec_convert_dtuple_to_rec(heap_top, tuple);
 
-	ut_ad(supremum_rec == page + PAGE_SUPREMUM);
+	ut_a(supremum_rec == page + PAGE_SUPREMUM);
 
 	rec_set_n_owned(supremum_rec, 1);
 	rec_set_heap_no(supremum_rec, 1);
@@ -389,6 +372,8 @@ page_create(
 	page_header_set_ptr(page, PAGE_FREE, NULL);
 	page_header_set_field(page, PAGE_GARBAGE, 0);
 	page_header_set_ptr(page, PAGE_LAST_INSERT, NULL);
+	page_header_set_field(page, PAGE_DIRECTION, PAGE_NO_DIRECTION);
+	page_header_set_field(page, PAGE_N_DIRECTION, 0);
 	page_header_set_field(page, PAGE_N_RECS, 0);
 	page_set_max_trx_id(page, ut_dulint_zero);
 	
@@ -402,17 +387,11 @@ page_create(
 	slot = page_dir_get_nth_slot(page, 1);
 	page_dir_slot_set_rec(slot, supremum_rec);
 
-	/* Set next pointers in infimum and supremum */
+	/* Set the next pointers in infimum and supremum */
 	
 	rec_set_next_offs(infimum_rec, (ulint)(supremum_rec - page)); 
 	rec_set_next_offs(supremum_rec, 0);
 
-	if (page_template == NULL) {
-		page_template = mem_alloc(UNIV_PAGE_SIZE);
-
-		ut_memcpy(page_template, page, UNIV_PAGE_SIZE);
-	}
-	
 	return(page);
 }
 
@@ -439,6 +418,8 @@ page_copy_rec_list_end_no_locks(
 		page_cur_move_to_next(&cur1);
 	}
 	
+	ut_a(mach_read_from_2(new_page + UNIV_PAGE_SIZE - 10) == PAGE_INFIMUM);
+
 	page_cur_set_before_first(new_page, &cur2);
 	
 	/* Copy records from the original page to the new page */	
@@ -446,8 +427,22 @@ page_copy_rec_list_end_no_locks(
 	sup = page_get_supremum_rec(page);
 	
 	while (sup != page_cur_get_rec(&cur1)) {
-		ut_a(
-		page_cur_rec_insert(&cur2, page_cur_get_rec(&cur1), mtr));
+		if (!page_cur_rec_insert(&cur2,
+					page_cur_get_rec(&cur1), mtr)) {
+			/* Track an assertion failure reported on the mailing
+			list on June 18th, 2003 */
+
+		        buf_page_print(new_page);
+		        buf_page_print(page);
+			ut_print_timestamp(stderr);
+
+			fprintf(stderr,
+"InnoDB: rec offset %lu, cur1 offset %lu, cur2 offset %lu\n",
+			      (ulint)(rec - page),
+			      (ulint)(page_cur_get_rec(&cur1) - page),
+			      (ulint)(page_cur_get_rec(&cur2) - new_page));
+			ut_error;
+		}
 
 		page_cur_move_to_next(&cur1);
 		page_cur_move_to_next(&cur2);
@@ -1125,8 +1120,8 @@ page_rec_print(
 /*===========*/
 	rec_t*	rec)
 {
-	rec_print(rec);
-	printf(
+	rec_print(stderr, rec);
+	fprintf(stderr,
      		"            n_owned: %lu; heap_no: %lu; next rec: %lu\n",
 		rec_get_n_owned(rec),
 		rec_get_heap_no(rec),
@@ -1152,25 +1147,26 @@ page_dir_print(
 
 	n = page_dir_get_n_slots(page);
 	
-	printf("--------------------------------\n");
-	printf("PAGE DIRECTORY\n");
-	printf("Page address %lx\n", (ulint)page);
-	printf("Directory stack top at offs: %lu; number of slots: %lu\n", 
-		(ulint)(page_dir_get_nth_slot(page, n - 1) - page), n);
+	fprintf(stderr, "--------------------------------\n"
+		"PAGE DIRECTORY\n"
+		"Page address %p\n"
+		"Directory stack top at offs: %lu; number of slots: %lu\n",
+		page, (ulint)(page_dir_get_nth_slot(page, n - 1) - page), n);
 	for (i = 0; i < n; i++) {
 		slot = page_dir_get_nth_slot(page, i);
 		if ((i == pr_n) && (i < n - pr_n)) {
-			printf("    ...   \n");
+			fputs("    ...   \n", stderr);
 		}
 	    	if ((i < pr_n) || (i >= n - pr_n)) {
-	   		printf(
+			fprintf(stderr,
 	   	   "Contents of slot: %lu: n_owned: %lu, rec offs: %lu\n",
 			i, page_dir_slot_get_n_owned(slot),
 			(ulint)(page_dir_slot_get_rec(slot) - page));
 	    	}
 	}
-	printf("Total of %lu records\n", 2 + page_get_n_recs(page));	
-	printf("--------------------------------\n");
+	fprintf(stderr, "Total of %lu records\n"
+		"--------------------------------\n",
+		2 + page_get_n_recs(page));
 }	
 	
 /*******************************************************************
@@ -1184,21 +1180,20 @@ page_print_list(
 	ulint	pr_n)	/* in: print n first and n last entries */
 {
 	page_cur_t	cur;
-	rec_t*		rec;
 	ulint		count;
 	ulint		n_recs;
 
-	printf("--------------------------------\n");
-	printf("PAGE RECORD LIST\n");
-	printf("Page address %lu\n", (ulint)page);
+	fprintf(stderr,
+		"--------------------------------\n"
+		"PAGE RECORD LIST\n"
+		"Page address %p\n", page);
 
 	n_recs = page_get_n_recs(page);
 
 	page_cur_set_before_first(page, &cur);
 	count = 0;
 	for (;;) {
-		rec = (&cur)->rec;
-		page_rec_print(rec);
+		page_rec_print(cur.rec);
 
 		if (count == pr_n) {
 			break;
@@ -1211,24 +1206,22 @@ page_print_list(
 	}
 	
 	if (n_recs > 2 * pr_n) {
-		printf(" ... \n");
+		fputs(" ... \n", stderr);
 	}
 	
-	for (;;) {
-		if (page_cur_is_after_last(&cur)) {
-			break;
-		}	
+	while (!page_cur_is_after_last(&cur)) {
 		page_cur_move_to_next(&cur);
 
 		if (count + pr_n >= n_recs) {	
-			rec = (&cur)->rec;
-			page_rec_print(rec);
+			page_rec_print(cur.rec);
 		}
 		count++;	
 	}
 
-	printf("Total of %lu records \n", count + 1);	
-	printf("--------------------------------\n");
+	fprintf(stderr,
+		"Total of %lu records \n"
+		"--------------------------------\n",
+		count + 1);
 }	
 
 /*******************************************************************
@@ -1239,21 +1232,19 @@ page_header_print(
 /*==============*/
 	page_t*	page)
 {
-	printf("--------------------------------\n");
-	printf("PAGE HEADER INFO\n");
-	printf("Page address %lx, n records %lu\n", (ulint)page,
-		page_header_get_field(page, PAGE_N_RECS));
-
-	printf("n dir slots %lu, heap top %lu\n",
+	fprintf(stderr,
+		"--------------------------------\n"
+		"PAGE HEADER INFO\n"
+		"Page address %p, n records %lu\n"
+		"n dir slots %lu, heap top %lu\n"
+		"Page n heap %lu, free %lu, garbage %lu\n"
+		"Page last insert %lu, direction %lu, n direction %lu\n",
+		page, page_header_get_field(page, PAGE_N_RECS),
 		page_header_get_field(page, PAGE_N_DIR_SLOTS),
-		page_header_get_field(page, PAGE_HEAP_TOP));
-
-	printf("Page n heap %lu, free %lu, garbage %lu\n",
+		page_header_get_field(page, PAGE_HEAP_TOP),
 		page_header_get_field(page, PAGE_N_HEAP),
 		page_header_get_field(page, PAGE_FREE),
-		page_header_get_field(page, PAGE_GARBAGE));
-
-	printf("Page last insert %lu, direction %lu, n direction %lu\n",
+		page_header_get_field(page, PAGE_GARBAGE),
 		page_header_get_field(page, PAGE_LAST_INSERT),
 		page_header_get_field(page, PAGE_DIRECTION),
 		page_header_get_field(page, PAGE_N_DIRECTION));
@@ -1299,19 +1290,242 @@ page_rec_validate(
 	heap_no = rec_get_heap_no(rec);
 
 	if (!(n_owned <= PAGE_DIR_SLOT_MAX_N_OWNED)) {
-		fprintf(stderr, "Dir slot n owned too big %lu\n", n_owned);
+		fprintf(stderr,
+			"InnoDB: Dir slot of rec %lu, n owned too big %lu\n",
+				(ulint)(rec - page), n_owned);
 		return(FALSE);
 	}
 
 	if (!(heap_no < page_header_get_field(page, PAGE_N_HEAP))) {
-		fprintf(stderr, "Heap no too big %lu %lu\n", heap_no,
+		fprintf(stderr,
+		"InnoDB: Heap no of rec %lu too big %lu %lu\n",
+				(ulint)(rec - page), heap_no,
 				page_header_get_field(page, PAGE_N_HEAP));
 		return(FALSE);
 	}
 	
 	return(TRUE);
 }
+
+/*******************************************************************
+Checks that the first directory slot points to the infimum record and
+the last to the supremum. This function is intended to track if the
+bug fixed in 4.0.14 has caused corruption to users' databases. */
+
+void
+page_check_dir(
+/*===========*/
+	page_t*	page)	/* in: index page */
+{
+	ulint	n_slots;
+
+	n_slots = page_dir_get_n_slots(page);
+
+	if (page_dir_slot_get_rec(page_dir_get_nth_slot(page, 0))
+	    != page_get_infimum_rec(page)) {
+
+	        fprintf(stderr,
+"InnoDB: Page directory corruption: supremum not pointed to\n");
+		buf_page_print(page);
+       	}
+
+	if (page_dir_slot_get_rec(page_dir_get_nth_slot(page, n_slots - 1))
+	    != page_get_supremum_rec(page)) {
+
+	        fprintf(stderr,
+"InnoDB: Page directory corruption: supremum not pointed to\n");
+		buf_page_print(page);
+       	}
+}
 	
+/*******************************************************************
+This function checks the consistency of an index page when we do not
+know the index. This is also resilient so that this should never crash
+even if the page is total garbage. */
+
+ibool
+page_simple_validate(
+/*=================*/
+			/* out: TRUE if ok */
+	page_t*	page)	/* in: index page */
+{
+	page_cur_t 	cur;
+	page_dir_slot_t* slot;
+	ulint		slot_no;
+	ulint		n_slots;
+	rec_t*		rec;
+	byte*		rec_heap_top;
+	ulint		count;
+	ulint		own_count;
+	ibool		ret	= FALSE;
+
+	/* Check first that the record heap and the directory do not
+	overlap. */
+
+	n_slots = page_dir_get_n_slots(page);
+
+	if (n_slots > UNIV_PAGE_SIZE / 4) {
+		fprintf(stderr,
+	"InnoDB: Nonsensical number %lu of page dir slots\n", n_slots);
+
+		goto func_exit;
+	}
+
+	rec_heap_top = page_header_get_ptr(page, PAGE_HEAP_TOP);
+	
+	if (rec_heap_top > page_dir_get_nth_slot(page, n_slots - 1)) {
+
+		fprintf(stderr,
+    "InnoDB: Record heap and dir overlap on a page, heap top %lu, dir %lu\n",
+       		(ulint)(page_header_get_ptr(page, PAGE_HEAP_TOP) - page),
+       		(ulint)(page_dir_get_nth_slot(page, n_slots - 1) - page));
+
+       		goto func_exit;
+       	}
+
+	/* Validate the record list in a loop checking also that it is
+	consistent with the page record directory. */
+
+	count = 0;
+	own_count = 1;
+	slot_no = 0;
+	slot = page_dir_get_nth_slot(page, slot_no);
+
+	page_cur_set_before_first(page, &cur);
+
+	for (;;) {
+		rec = (&cur)->rec;
+		
+		if (rec > rec_heap_top) {
+			fprintf(stderr,
+			"InnoDB: Record %lu is above rec heap top %lu\n",
+			(ulint)(rec - page), (ulint)(rec_heap_top - page));
+
+			goto func_exit;
+		}
+
+		if (rec_get_n_owned(rec) != 0) {
+			/* This is a record pointed to by a dir slot */
+			if (rec_get_n_owned(rec) != own_count) {
+
+				fprintf(stderr,
+		"InnoDB: Wrong owned count %lu, %lu, rec %lu\n",
+				rec_get_n_owned(rec), own_count,
+				(ulint)(rec - page));
+
+				goto func_exit;
+			}
+
+			if (page_dir_slot_get_rec(slot) != rec) {
+				fprintf(stderr,
+		"InnoDB: Dir slot does not point to right rec %lu\n",
+					(ulint)(rec - page));
+
+				goto func_exit;
+			}
+						
+			own_count = 0;
+
+			if (!page_cur_is_after_last(&cur)) {
+				slot_no++;
+				slot = page_dir_get_nth_slot(page, slot_no);
+			}
+		}
+
+		if (page_cur_is_after_last(&cur)) {
+
+			break;
+		}
+
+		if (rec_get_next_offs(rec) < FIL_PAGE_DATA
+				|| rec_get_next_offs(rec) >= UNIV_PAGE_SIZE) {
+			fprintf(stderr,
+		"InnoDB: Next record offset nonsensical %lu for rec %lu\n",
+			  rec_get_next_offs(rec),
+			  (ulint)(rec - page));
+
+			goto func_exit;
+		}
+
+		count++;		
+
+		if (count > UNIV_PAGE_SIZE) {
+			fprintf(stderr,
+		"InnoDB: Page record list appears to be circular %lu\n",
+								count);
+			goto func_exit;
+		}
+		
+		page_cur_move_to_next(&cur);
+		own_count++;
+	}
+	
+	if (rec_get_n_owned(rec) == 0) {
+		fprintf(stderr, "InnoDB: n owned is zero in a supremum rec\n");
+
+		goto func_exit;
+	}
+		
+	if (slot_no != n_slots - 1) {
+		fprintf(stderr, "InnoDB: n slots wrong %lu, %lu\n",
+			slot_no, n_slots - 1);
+		goto func_exit;
+	}		
+
+	if (page_header_get_field(page, PAGE_N_RECS) + 2 != count + 1) {
+		fprintf(stderr, "InnoDB: n recs wrong %lu %lu\n",
+		page_header_get_field(page, PAGE_N_RECS) + 2,  count + 1);
+
+		goto func_exit;
+	}
+
+	/* Check then the free list */
+	rec = page_header_get_ptr(page, PAGE_FREE);
+
+	while (rec != NULL) {
+		if (rec < page + FIL_PAGE_DATA
+				|| rec >= page + UNIV_PAGE_SIZE) {
+			fprintf(stderr,
+		"InnoDB: Free list record has a nonsensical offset %lu\n",
+			(ulint)(rec - page));
+
+			goto func_exit;
+		}
+
+		if (rec > rec_heap_top) {
+			fprintf(stderr,
+		"InnoDB: Free list record %lu is above rec heap top %lu\n",
+			(ulint)(rec - page), (ulint)(rec_heap_top - page));
+
+			goto func_exit;
+		}
+
+		count++;
+		
+		if (count > UNIV_PAGE_SIZE) {
+			fprintf(stderr,
+		"InnoDB: Page free list appears to be circular %lu\n",
+								count);
+			goto func_exit;
+		}
+
+		rec = page_rec_get_next(rec);
+	}
+	
+	if (page_header_get_field(page, PAGE_N_HEAP) != count + 1) {
+
+		fprintf(stderr, "InnoDB: N heap is wrong %lu, %lu\n",
+		page_header_get_field(page, PAGE_N_HEAP), count + 1);
+
+		goto func_exit;
+	}
+
+	ret = TRUE;	
+
+func_exit:
+	return(ret);			  
+}
+
 /*******************************************************************
 This function checks the consistency of an index page. */
 
@@ -1337,8 +1551,11 @@ page_validate(
 	ulint		n_slots;
 	ibool		ret	= FALSE;
 	ulint		i;
-	char           	err_buf[1000];
 	
+	if (!page_simple_validate(page)) {
+		goto func_exit2;
+	}
+
 	heap = mem_heap_create(UNIV_PAGE_SIZE);
 	
 	/* The following buffer is used to check that the
@@ -1356,10 +1573,13 @@ page_validate(
 
 	if (!(page_header_get_ptr(page, PAGE_HEAP_TOP) <=
 			page_dir_get_nth_slot(page, n_slots - 1))) {
-		fprintf(stderr,
-       	"Record heap and dir overlap on a page in index %s, %lu, %lu\n",
-       		index->name, (ulint)page_header_get_ptr(page, PAGE_HEAP_TOP),
-       		(ulint)page_dir_get_nth_slot(page, n_slots - 1));
+
+		fputs("InnoDB: Record heap and dir overlap on a page ",
+			stderr);
+		dict_index_name_print(stderr, index);
+		fprintf(stderr, ", %p, %p\n",
+				page_header_get_ptr(page, PAGE_HEAP_TOP),
+			page_dir_get_nth_slot(page, n_slots - 1));
 
        		goto func_exit;
        	}
@@ -1375,7 +1595,7 @@ page_validate(
 	page_cur_set_before_first(page, &cur);
 
 	for (;;) {
-		rec = (&cur)->rec;
+		rec = cur.rec;
 
 		if (!page_rec_validate(rec)) {
 			goto func_exit;
@@ -1385,13 +1605,14 @@ page_validate(
 		if ((count >= 2) && (!page_cur_is_after_last(&cur))) {
 			if (!(1 == cmp_rec_rec(rec, old_rec, index))) {
 				fprintf(stderr,
-					"Records in wrong order in index %s\n",
-					index->name);
-	 		 	rec_sprintf(err_buf, 900, old_rec);
-	  			fprintf(stderr, "InnoDB: record %s\n", err_buf);
-				
-	 		 	rec_sprintf(err_buf, 900, rec);
-	  			fprintf(stderr, "InnoDB: record %s\n", err_buf);
+				"InnoDB: Records in wrong order on page %lu",
+					buf_frame_get_page_no(page));
+				dict_index_name_print(stderr, index);
+				fputs("\nInnoDB: previous record ", stderr);
+				rec_print(stderr, old_rec);
+				fputs("\nInnoDB: record ", stderr);
+				rec_print(stderr, rec);
+				putc('\n', stderr);
 				
 				goto func_exit;
 			}
@@ -1409,10 +1630,8 @@ page_validate(
 			if (!buf[offs + i] == 0) {
 				/* No other record may overlap this */
 
-				fprintf(stderr,
-				"Record overlaps another in index %s \n",
-				index->name);
-
+				fputs("InnoDB: Record overlaps another\n",
+					stderr);
 				goto func_exit;
 			}
 				
@@ -1423,18 +1642,15 @@ page_validate(
 			/* This is a record pointed to by a dir slot */
 			if (rec_get_n_owned(rec) != own_count) {
 				fprintf(stderr,
-				"Wrong owned count %lu, %lu, in index %s\n",
-				rec_get_n_owned(rec), own_count,
-				index->name);
-
+			"InnoDB: Wrong owned count %lu, %lu\n",
+				rec_get_n_owned(rec), own_count);
 				goto func_exit;
 			}
 
 			if (page_dir_slot_get_rec(slot) != rec) {
-				fprintf(stderr,
-				"Dir slot does not point to right rec in %s\n",
-				index->name);
-
+				fputs(
+			"InnoDB: Dir slot does not point to right rec\n",
+					stderr);
 				goto func_exit;
 			}
 			
@@ -1454,9 +1670,8 @@ page_validate(
 		if (rec_get_next_offs(rec) < FIL_PAGE_DATA
 				|| rec_get_next_offs(rec) >= UNIV_PAGE_SIZE) {
 			fprintf(stderr,
-			  "Next record offset wrong %lu in index %s\n",
-			  rec_get_next_offs(rec), index->name);
-
+				"InnoDB: Next record offset wrong %lu\n",
+				rec_get_next_offs(rec));
 			goto func_exit;
 		}
 
@@ -1467,27 +1682,25 @@ page_validate(
 	}
 	
 	if (rec_get_n_owned(rec) == 0) {
-		fprintf(stderr, "n owned is zero in index %s\n", index->name);
-
+		fputs("InnoDB: n owned is zero\n", stderr);
 		goto func_exit;
 	}
 		
 	if (slot_no != n_slots - 1) {
-		fprintf(stderr, "n slots wrong %lu %lu in index %s\n",
-			slot_no, n_slots - 1, index->name);
+		fprintf(stderr, "InnoDB: n slots wrong %lu %lu\n",
+			slot_no, n_slots - 1);
 		goto func_exit;
 	}		
 
 	if (page_header_get_field(page, PAGE_N_RECS) + 2 != count + 1) {
-		fprintf(stderr, "n recs wrong %lu %lu in index %s\n",
-		page_header_get_field(page, PAGE_N_RECS) + 2,  count + 1,
-		index->name);
-
+		fprintf(stderr, "InnoDB: n recs wrong %lu %lu\n",
+		page_header_get_field(page, PAGE_N_RECS) + 2,  count + 1);
 		goto func_exit;
 	}
 
 	if (data_size != page_get_data_size(page)) {
-		fprintf(stderr, "Summed data size %lu, returned by func %lu\n",
+		fprintf(stderr,
+		"InnoDB: Summed data size %lu, returned by func %lu\n",
 			data_size, page_get_data_size(page));
 		goto func_exit;
 	}
@@ -1507,10 +1720,8 @@ page_validate(
 		for (i = 0; i < rec_get_size(rec); i++) {
 
 			if (buf[offs + i] != 0) {
-				fprintf(stderr,
-	                "Record overlaps another in free list, index %s \n",
-				index->name);
-
+				fputs(
+		"InnoDB: Record overlaps another in free list\n", stderr);
 				goto func_exit;
 			}
 				
@@ -1521,10 +1732,9 @@ page_validate(
 	}
 	
 	if (page_header_get_field(page, PAGE_N_HEAP) != count + 1) {
-
-		fprintf(stderr, "N heap is wrong %lu %lu in index %s\n",
-		page_header_get_field(page, PAGE_N_HEAP), count + 1,
-		index->name);
+		fprintf(stderr, "InnoDB: N heap is wrong %lu %lu\n",
+			page_header_get_field(page, PAGE_N_HEAP), count + 1);
+		goto func_exit;
 	}
 
 	ret = TRUE;	
@@ -1532,6 +1742,15 @@ page_validate(
 func_exit:
 	mem_heap_free(heap);
 
+	if (ret == FALSE) {
+	func_exit2:
+		fprintf(stderr, "InnoDB: Apparent corruption in page %lu in ",
+			buf_frame_get_page_no(page));
+		dict_index_name_print(stderr, index);
+		putc('\n', stderr);
+		buf_page_print(page);
+	}
+	
 	return(ret);			  
 }
 
@@ -1546,16 +1765,13 @@ page_find_rec_with_heap_no(
 	ulint	heap_no)/* in: heap number */
 {
 	page_cur_t	cur;
-	rec_t*		rec;
 
 	page_cur_set_before_first(page, &cur);
 
 	for (;;) {
-		rec = (&cur)->rec;
+		if (rec_get_heap_no(cur.rec) == heap_no) {
 
-		if (rec_get_heap_no(rec) == heap_no) {
-
-			return(rec);
+			return(cur.rec);
 		}
 
 		if (page_cur_is_after_last(&cur)) {
