@@ -1094,6 +1094,19 @@ opt_clust_access(
 	for (i = 0; i < n_fields; i++) {
 		pos = dict_index_get_nth_field_pos(index, clust_index, i);
 
+		ut_a(pos != ULINT_UNDEFINED);
+
+		/* We optimize here only queries to InnoDB's internal system
+		tables, and they should not contain column prefix indexes. */
+
+		if (dict_index_get_nth_field(index, pos)->prefix_len != 0
+		    || dict_index_get_nth_field(clust_index, i)
+							->prefix_len != 0) {
+			fprintf(stderr,
+"InnoDB: Error in pars0opt.c: table %s has prefix_len != 0\n",
+				index->table_name);
+		}
+
 		*(plan->clust_map + i) = pos;
 
 		ut_ad((pos != ULINT_UNDEFINED)
@@ -1190,7 +1203,6 @@ opt_search_plan(
 #endif
 }
 
-#ifdef UNIV_SQL_DEBUG
 /************************************************************************
 Prints info of a query plan. */
 
@@ -1236,4 +1248,3 @@ opt_print_query_plan(
 			UT_LIST_GET_LEN(plan->end_conds));
 	}
 }
-#endif /* UNIV_SQL_DEBUG */
