@@ -32,7 +32,6 @@ Created 10/8/1995 Heikki Tuuri
 #include "mem0mem.h"
 #include "mem0pool.h"
 #include "sync0sync.h"
-#include "sync0ipm.h"
 #include "thr0loc.h"
 #include "que0que.h"
 #include "srv0que.h"
@@ -56,6 +55,13 @@ ibool	srv_lower_case_table_names	= FALSE;
 in the server */
 ulint	srv_activity_count	= 0;
 
+/* The following is the maximum allowed duration of a lock wait. */
+ulint	srv_fatal_semaphore_wait_threshold = 600;
+
+/* How much data manipulation language (DML) statements need to be delayed,
+in microseconds, in order to reduce the lagging of the purge thread. */
+ulint	srv_dml_needed_delay = 0;
+
 ibool	srv_lock_timeout_and_monitor_active = FALSE;
 ibool	srv_error_monitor_active = FALSE;
 
@@ -221,6 +227,9 @@ merge to completion before shutdown */
 
 ibool	srv_fast_shutdown	= FALSE;
 
+/* Generate a innodb_status.<pid> file */
+ibool	srv_innodb_status	= FALSE;
+
 ibool	srv_use_doublewrite_buf	= TRUE;
 
 ibool   srv_set_thread_priorities = TRUE;
@@ -836,6 +845,8 @@ srv_general_init(void)
 
 /*======================= InnoDB Server FIFO queue =======================*/
 
+/* Maximum allowable purge history length.  <=0 means 'infinite'. */
+ulint	srv_max_purge_lag		= 0;
 
 /*************************************************************************
 Puts an OS thread to wait if there are too many concurrent threads
@@ -1428,7 +1439,6 @@ srv_printf_innodb_monitor(
 
 	srv_last_monitor_time = time(NULL);
 
-	rewind(file);
 	fputs("\n=====================================\n", file);
 
 	ut_print_timestamp(file);
@@ -1606,11 +1616,13 @@ loop:
 		srv_printf_innodb_monitor(stderr);
 	    }
 
-	    mutex_enter(&srv_monitor_file_mutex);
-	    rewind(srv_monitor_file);
-	    srv_printf_innodb_monitor(srv_monitor_file);
-	    os_file_set_eof(srv_monitor_file);
-	    mutex_exit(&srv_monitor_file_mutex);
+	    if (srv_innodb_status) {
+		mutex_enter(&srv_monitor_file_mutex);
+		rewind(srv_monitor_file);
+		srv_printf_innodb_monitor(srv_monitor_file);
+		os_file_set_eof(srv_monitor_file);
+		mutex_exit(&srv_monitor_file_mutex);
+	    }
 
 	    if (srv_print_innodb_tablespace_monitor
 		&& difftime(current_time, last_table_monitor_time) > 60) {
@@ -1749,7 +1761,8 @@ srv_error_monitor_thread(
 			/* in: a dummy parameter required by
 			os_thread_create */
 {
-	ulint	cnt	= 0;
+	/* number of successive fatal timeouts observed */
+	ulint	fatal_cnt	= 0;
 
 #ifdef UNIV_DEBUG_THREAD_CREATION
 	fprintf(stderr, "Error monitor thread starts, id %lu\n",
@@ -1758,8 +1771,6 @@ srv_error_monitor_thread(
 loop:
 	srv_error_monitor_active = TRUE;
 
-	cnt++;
-
 	os_thread_sleep(2000000);
 
 	if (difftime(time(NULL), srv_last_monitor_time) > 60) {
@@ -1769,7 +1780,20 @@ loop:
 		srv_refresh_innodb_monitor_stats();
 	}
 
-	sync_array_print_long_waits();
+	if (sync_array_print_long_waits()) {
+		fatal_cnt++;
+		if (fatal_cnt > 5) {
+
+			fprintf(stderr,
+"InnoDB: Error: semaphore wait has lasted > %lu seconds\n"
+"InnoDB: We intentionally crash the server, because it appears to be hung.\n",
+				srv_fatal_semaphore_wait_threshold);
+
+			ut_error;
+		}
+	} else {
+		fatal_cnt = 0;
+	}
 
 	/* Flush stderr so that a database user gets the output
 	to possible MySQL error file */
