@@ -61,10 +61,7 @@ struct sync_cell_struct {
 					thread */
 	ibool		waiting;	/* TRUE if the thread has already
 					called sync_array_event_wait
-					on this cell but not yet
-					sync_array_free_cell (which
-					actually resets wait_object and thus
-					whole cell) */
+					on this cell */
 	ibool		event_set;	/* TRUE if the event is set */
         os_event_t 	event;   	/* operating system event
                                         semaphore handle */
@@ -449,7 +446,6 @@ sync_array_cell_print(
 {
 	mutex_t*	mutex;
 	rw_lock_t*	rwlock;
-	char*		str __attribute__((unused)) = NULL;
 	ulint		type;
 
 	type = cell->request_type;
@@ -891,16 +887,20 @@ sync_arr_wake_threads_if_sema_free(void)
 }
 
 /**************************************************************************
-Prints warnings of long semaphore waits to stderr. Currently > 120 sec. */
+Prints warnings of long semaphore waits to stderr. */
 
-void
+ibool
 sync_array_print_long_waits(void)
 /*=============================*/
+			/* out: TRUE if fatal semaphore wait threshold
+			was exceeded */
 {
         sync_cell_t*   	cell;
         ibool		old_val;
 	ibool		noticed = FALSE;
 	ulint           i;
+	ulint		fatal_timeout = srv_fatal_semaphore_wait_threshold;
+	ibool		fatal = FALSE;
 
         for (i = 0; i < sync_primary_wait_array->n_cells; i++) {
 
@@ -915,14 +915,9 @@ sync_array_print_long_waits(void)
                 }
 
                 if (cell->wait_object != NULL
-		    && difftime(time(NULL), cell->reservation_time) > 600) {
-
-			fputs(
-"InnoDB: Error: semaphore wait has lasted > 600 seconds\n"
-"InnoDB: We intentionally crash the server, because it appears to be hung.\n",
-				stderr);
-
-		    	ut_error;
+		    && difftime(time(NULL), cell->reservation_time)
+		    > fatal_timeout) {
+			fatal = TRUE;
                 }
        	}
 
@@ -950,6 +945,8 @@ sync_array_print_long_waits(void)
 		fprintf(stderr,
 "InnoDB: ###### Diagnostic info printed to the standard error stream\n");
 	}
+
+	return(fatal);
 }
 
 /**************************************************************************
