@@ -390,14 +390,13 @@ trx_undo_seg_create(
 	ibool		success;
 	
 	ut_ad(mtr && id && rseg_hdr);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
-/*	
-	if (type == TRX_UNDO_INSERT) {
-		printf("Creating insert undo log segment\n");
-	} else {
-		printf("Creating update undo log segment\n");
-	}
-*/
+#endif /* UNIV_SYNC_DEBUG */
+
+/*	fputs(type == TRX_UNDO_INSERT
+		? "Creating insert undo log segment\n"
+		: "Creating update undo log segment\n", stderr); */
 	slot_no = trx_rsegf_undo_find_free(rseg_hdr, mtr);
 
 	if (slot_no == ULINT_UNDEFINED) {
@@ -430,7 +429,9 @@ trx_undo_seg_create(
 		return(NULL);
 	}
 
+#ifdef UNIV_SYNC_DEBUG
 	buf_page_dbg_add_level(undo_page, SYNC_TRX_UNDO_PAGE);
+#endif /* UNIV_SYNC_DEBUG */
 
 	page_hdr = undo_page + TRX_UNDO_PAGE_HDR;
 	seg_hdr = undo_page + TRX_UNDO_SEG_HDR;
@@ -735,13 +736,14 @@ trx_undo_add_page(
 	ulint		page_no;
 	ibool		success;
 	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(trx->undo_mutex)));
 	ut_ad(!mutex_own(&kernel_mutex));
+	ut_ad(mutex_own(&(trx->rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	rseg = trx->rseg;
 
-	ut_ad(mutex_own(&(rseg->mutex)));
-
 	if (rseg->curr_size == rseg->max_size) {
 
 		return(FIL_NULL);
@@ -811,8 +813,10 @@ trx_undo_free_page(
 
 	UT_NOT_USED(hdr_offset);
 	ut_a(hdr_page_no != page_no);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(!mutex_own(&kernel_mutex));
 	ut_ad(mutex_own(&(rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	
 	undo_page = trx_undo_page_get(space, page_no, mtr);
 
@@ -859,7 +863,9 @@ trx_undo_free_page_in_rollback(
 	ulint	last_page_no;
 
 	ut_ad(undo->hdr_page_no != page_no);
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(trx->undo_mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	last_page_no = trx_undo_free_page(undo->rseg, FALSE, undo->space,
 					undo->hdr_page_no, undo->hdr_offset,
@@ -913,12 +919,13 @@ trx_undo_truncate_end(
 	trx_rseg_t*	rseg;
 	mtr_t		mtr;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(trx->undo_mutex)));
+	ut_ad(mutex_own(&(trx->rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	rseg = trx->rseg;		
 
-	ut_ad(mutex_own(&(rseg->mutex)));
-
  	for (;;) {
 		mtr_start(&mtr);
 
@@ -992,7 +999,9 @@ trx_undo_truncate_start(
 	ulint		page_no;
 	mtr_t		mtr;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (0 == ut_dulint_cmp(limit, ut_dulint_zero)) {
 
@@ -1058,8 +1067,9 @@ trx_undo_seg_free(
 	while (!finished) {
 
 		mtr_start(&mtr);
-	
+#ifdef UNIV_SYNC_DEBUG
 		ut_ad(!mutex_own(&kernel_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 		mutex_enter(&(rseg->mutex));
 
 		seg_header = trx_undo_page_get(undo->space, undo->hdr_page_no,
@@ -1115,7 +1125,7 @@ trx_undo_mem_create_at_db_start(
 	if (id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr,
 		"InnoDB: Error: undo->id is %lu\n", id);
-		ut_a(0);
+		ut_error;
 	}
 
 	undo_page = trx_undo_page_get(rseg->space, page_no, mtr);
@@ -1132,8 +1142,7 @@ trx_undo_mem_create_at_db_start(
 	
 	undo_header = undo_page + offset;
 
-	trx_id = mtr_read_dulint(undo_header + TRX_UNDO_TRX_ID, MLOG_8BYTES,
-									mtr);
+	trx_id = mtr_read_dulint(undo_header + TRX_UNDO_TRX_ID, mtr);
 	mutex_enter(&(rseg->mutex));
 
 	undo = trx_undo_mem_create(rseg, id, type, trx_id, page_no, offset);
@@ -1143,8 +1152,7 @@ trx_undo_mem_create_at_db_start(
 	undo->dict_operation = mtr_read_ulint(
 				undo_header + TRX_UNDO_DICT_OPERATION,
 							MLOG_2BYTES, mtr);
-	undo->table_id = mtr_read_dulint(undo_header + TRX_UNDO_TABLE_ID,
-							MLOG_8BYTES, mtr);
+	undo->table_id = mtr_read_dulint(undo_header + TRX_UNDO_TABLE_ID, mtr);
 	undo->state = state;
 	undo->size = flst_get_len(seg_header + TRX_UNDO_PAGE_LIST, mtr);
 
@@ -1268,12 +1276,14 @@ trx_undo_mem_create(
 {
 	trx_undo_t*	undo;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr,
 		"InnoDB: Error: undo->id is %lu\n", id);
-		ut_a(0);
+		ut_error;
 	}
 
 	undo = mem_alloc(sizeof(trx_undo_t));
@@ -1312,13 +1322,15 @@ trx_undo_mem_init_for_reuse(
 				is created */
 	ulint		offset)	/* in: undo log header byte offset on page */
 {
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&((undo->rseg)->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 	
  	if (undo->id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr, "InnoDB: Error: undo->id is %lu\n", undo->id);
 
 		mem_analyze_corruption((byte*)undo);
-		ut_a(0);
+		ut_error;
 	}
 
 	undo->state = TRX_UNDO_ACTIVE;
@@ -1342,7 +1354,7 @@ trx_undo_mem_free(
 	if (undo->id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr,
 		"InnoDB: Error: undo->id is %lu\n", undo->id);
-		ut_a(0);
+		ut_error;
 	}
 
 	mem_free(undo);
@@ -1370,7 +1382,9 @@ trx_undo_create(
 	trx_undo_t*	undo;
 	page_t*		undo_page;
 	
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (rseg->curr_size == rseg->max_size) {
 
@@ -1421,7 +1435,9 @@ trx_undo_reuse_cached(
 	page_t*		undo_page;
 	ulint		offset;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	if (type == TRX_UNDO_INSERT) {
 
@@ -1445,12 +1461,11 @@ trx_undo_reuse_cached(
 	}
 
 	ut_ad(undo->size == 1);
-	ut_ad(undo->hdr_page_no == undo->top_page_no);
 
 	if (undo->id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr, "InnoDB: Error: undo->id is %lu\n", undo->id);
 		mem_analyze_corruption((byte*)undo);
-		ut_a(0);
+		ut_error;
 	}
 
 	undo_page = trx_undo_page_get(undo->space, undo->hdr_page_no, mtr);
@@ -1491,7 +1506,7 @@ trx_undo_mark_as_dict_operation(
 			trx->dict_operation, MLOG_2BYTES, mtr);
 	
 	mlog_write_dulint(hdr_page + undo->hdr_offset + TRX_UNDO_TABLE_ID,
-			trx->table_id, MLOG_8BYTES, mtr);	
+			trx->table_id, mtr);	
 
 	undo->dict_operation = trx->dict_operation;
 	undo->table_id = trx->table_id;
@@ -1518,11 +1533,15 @@ trx_undo_assign_undo(
 
 	rseg = trx->rseg;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(trx->undo_mutex)));
+#endif /* UNIV_SYNC_DEBUG */
 
 	mtr_start(&mtr);
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(!mutex_own(&kernel_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 	mutex_enter(&(rseg->mutex));
 
 	undo = trx_undo_reuse_cached(rseg, type, trx->id, &mtr);
@@ -1582,7 +1601,7 @@ trx_undo_set_state_at_finish(
 	if (undo->id >= TRX_RSEG_N_SLOTS) {
 		fprintf(stderr, "InnoDB: Error: undo->id is %lu\n", undo->id);
 		mem_analyze_corruption((byte*)undo);
-		ut_a(0);
+		ut_error;
 	}
 
 	undo_page = trx_undo_page_get(undo->space, undo->hdr_page_no, mtr);
@@ -1627,8 +1646,9 @@ trx_undo_update_cleanup(
 	undo = trx->update_undo;
 	rseg = trx->rseg;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
-	
+#endif /* UNIV_SYNC_DEBUG */
 	trx_purge_add_update_undo_to_history(trx, undo_page, mtr);
 
 	UT_LIST_REMOVE(undo_list, rseg->update_undo_list, undo);
@@ -1667,8 +1687,10 @@ trx_undo_update_cleanup_by_discard(
 	undo = trx->update_undo;
 	rseg = trx->rseg;
 
+#ifdef UNIV_SYNC_DEBUG
 	ut_ad(mutex_own(&(rseg->mutex)));
 	ut_ad(mutex_own(&kernel_mutex));
+#endif /* UNIV_SYNC_DEBUG */
 	ut_ad(undo->size == 1);
 	ut_ad(undo->del_marks == FALSE);	
 	ut_ad(UT_LIST_GET_LEN(trx_sys->view_list) == 1);
