@@ -1,9 +1,12 @@
-/* Copyright (C) 2000-2003 MySQL AB
+/* Copyright (C) 2000-2004 MySQL AB
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+   the Free Software Foundation.
+
+   There are special exceptions to the terms and conditions of the GPL as it
+   is applied to this software. View the full text of the exception in file
+   EXCEPTIONS-CLIENT in the directory of this software distribution.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -59,6 +62,8 @@
 #define INADDR_NONE	-1
 #endif
 
+#undef net_buffer_length
+#undef max_allowed_packet
 static my_bool	mysql_client_init=0;
 uint		mysql_port=0;
 my_string	mysql_unix_port=0;
@@ -121,6 +126,14 @@ void STDCALL mysql_server_end()
   mysql_client_init= org_my_init_done= 0;
 }
 
+static MYSQL_PARAMETERS mysql_internal_parameters=
+{&max_allowed_packet, &net_buffer_length};
+
+MYSQL_PARAMETERS *STDCALL mysql_get_parameters(void)
+{
+  return &mysql_internal_parameters;
+}
+
 my_bool STDCALL mysql_thread_init()
 {
 #ifdef THREAD
@@ -931,7 +944,7 @@ static void mysql_read_default_options(struct st_mysql_options *options,
 	    options->client_flag&= ~CLIENT_LOCAL_FILES;
 	  break;
 	case 22:
-	  options->client_flag&= CLIENT_LOCAL_FILES;
+	  options->client_flag&= ~CLIENT_LOCAL_FILES;
           break;
 	case 23:  /* replication probe */
 	  options->rpl_probe= 1;
@@ -1631,7 +1644,7 @@ static int check_license(MYSQL *mysql)
   MYSQL_RES *res;
   NET *net= &mysql->net;
   static const char query[]= "SELECT @@license";
-  static const char required_license[]= LICENSE;
+  static const char required_license[]= STRINGIFY_ARG(LICENSE);
 
   if (mysql_real_query(mysql, query, sizeof(query)-1))
   {
@@ -1855,7 +1868,8 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
 	sprintf(net->last_error, ER(CR_UNKNOWN_HOST), host, tmp_errno);
 	goto error;
       }
-      memcpy(&sock_addr.sin_addr,hp->h_addr, (size_t) hp->h_length);
+      memcpy(&sock_addr.sin_addr, hp->h_addr,
+             min(sizeof(sock_addr.sin_addr), (size_t) hp->h_length));
       my_gethostbyname_r_free();
     }
     sock_addr.sin_port = (ushort) htons((ushort) port);
@@ -3139,6 +3153,41 @@ void my_net_local_init(NET *net)
   net->max_packet_size= max(net_buffer_length, max_allowed_packet);
 }
 
+/*
+  This function is used to create HEX string that you
+  can use in a SQL statement in of the either ways:
+    INSERT INTO blob_column VALUES (0xAABBCC);  (any MySQL version)
+    INSERT INTO blob_column VALUES (X'AABBCC'); (4.1 and higher)
+  
+  The string in "from" is encoded to a HEX string.
+  The result is placed in "to" and a terminating null byte is appended.
+  
+  The string pointed to by "from" must be "length" bytes long.
+  You must allocate the "to" buffer to be at least length*2+1 bytes long.
+  Each character needs two bytes, and you need room for the terminating
+  null byte. When mysql_hex_string() returns, the contents of "to" will
+  be a null-terminated string. The return value is the length of the
+  encoded string, not including the terminating null character.
+
+  The return value does not contain any leading 0x or a leading X' and
+  trailing '. The caller must supply whichever of those is desired.
+*/
+
+ulong STDCALL
+mysql_hex_string(char *to, const char *from, ulong length)
+{
+  char *to0= to;
+  const char *end;
+            
+  for (end= from + length; from < end; from++)
+  {
+    *to++= _dig_vec[((unsigned char) *from) >> 4];
+    *to++= _dig_vec[((unsigned char) *from) & 0x0F];
+  }
+  *to= '\0';
+  return (ulong) (to-to0);
+}
+
 /*
   Add escape characters to a string (blob?) to make it suitable for a insert
   to should at least have place for length*2+1 chars
