@@ -330,7 +330,7 @@ void Item_func_concat::fix_length_and_dec()
 String *Item_func_des_encrypt::val_str(String *str)
 {
 #ifdef HAVE_OPENSSL
-  des_cblock ivec;
+  DES_cblock ivec;
   struct st_des_keyblock keyblock;
   struct st_des_keyschedule keyschedule;
   const char *append_str="********";
@@ -370,9 +370,9 @@ String *Item_func_des_encrypt::val_str(String *str)
     EVP_BytesToKey(EVP_des_ede3_cbc(),EVP_md5(),NULL,
 		   (uchar*) keystr->ptr(), (int) keystr->length(),
 		   1, (uchar*) &keyblock,ivec);
-    des_set_key_unchecked(&keyblock.key1,keyschedule.ks1);
-    des_set_key_unchecked(&keyblock.key2,keyschedule.ks2);
-    des_set_key_unchecked(&keyblock.key3,keyschedule.ks3);
+    DES_set_key_unchecked(&keyblock.key1,&keyschedule.ks1);
+    DES_set_key_unchecked(&keyblock.key2,&keyschedule.ks2);
+    DES_set_key_unchecked(&keyblock.key3,&keyschedule.ks3);
   }
 
   /* 
@@ -393,12 +393,12 @@ String *Item_func_des_encrypt::val_str(String *str)
   tmp_value[0]=(char) (128 | key_number);
   // Real encryption
   bzero((char*) &ivec,sizeof(ivec));
-  des_ede3_cbc_encrypt((const uchar*) (res->ptr()),
+  DES_ede3_cbc_encrypt((const uchar*) (res->ptr()),
 		       (uchar*) (tmp_value.ptr()+1),
 		       res_length,
-		       keyschedule.ks1,
-		       keyschedule.ks2,
-		       keyschedule.ks3,
+		       &keyschedule.ks1,
+		       &keyschedule.ks2,
+		       &keyschedule.ks3,
 		       &ivec, TRUE);
   return &tmp_value;
 
@@ -412,8 +412,8 @@ error:
 String *Item_func_des_decrypt::val_str(String *str)
 {
 #ifdef HAVE_OPENSSL
-  des_key_schedule ks1, ks2, ks3;
-  des_cblock ivec;
+  DES_key_schedule ks1, ks2, ks3;
+  DES_cblock ivec;
   struct st_des_keyblock keyblock;
   struct st_des_keyschedule keyschedule;
   String *res= args[0]->val_str(str);
@@ -447,20 +447,20 @@ String *Item_func_des_decrypt::val_str(String *str)
 		   (uchar*) keystr->ptr(),(int) keystr->length(),
 		   1,(uchar*) &keyblock,ivec);
     // Here we set all 64-bit keys (56 effective) one by one
-    des_set_key_unchecked(&keyblock.key1,keyschedule.ks1);
-    des_set_key_unchecked(&keyblock.key2,keyschedule.ks2);
-    des_set_key_unchecked(&keyblock.key3,keyschedule.ks3); 
+    DES_set_key_unchecked(&keyblock.key1,&keyschedule.ks1);
+    DES_set_key_unchecked(&keyblock.key2,&keyschedule.ks2);
+    DES_set_key_unchecked(&keyblock.key3,&keyschedule.ks3); 
   }
   if (tmp_value.alloc(length-1))
     goto error;
 
   bzero((char*) &ivec,sizeof(ivec));
-  des_ede3_cbc_encrypt((const uchar*) res->ptr()+1,
+  DES_ede3_cbc_encrypt((const uchar*) res->ptr()+1,
 		       (uchar*) (tmp_value.ptr()),
 		       length-1,
-		       keyschedule.ks1,
-		       keyschedule.ks2,
-		       keyschedule.ks3,
+		       &keyschedule.ks1,
+		       &keyschedule.ks2,
+		       &keyschedule.ks3,
 		       &ivec, FALSE);
   /* Restore old length of key */
   if ((tail=(uint) (uchar) tmp_value[length-2]) > 8)
@@ -1135,7 +1135,7 @@ String *Item_func_ltrim::val_str(String *str)
   {
     const char *r_ptr=remove_str->ptr();
     end-=remove_length;
-    while (ptr < end && !memcmp(ptr,r_ptr,remove_length))
+    while (ptr <= end && !memcmp(ptr, r_ptr, remove_length))
       ptr+=remove_length;
     end+=remove_length;
   }
@@ -1206,8 +1206,8 @@ String *Item_func_rtrim::val_str(String *str)
     else
 #endif /* USE_MB */
     {
-      while (ptr + remove_length < end &&
-	     !memcmp(end-remove_length,r_ptr,remove_length))
+      while (ptr + remove_length <= end &&
+	     !memcmp(end-remove_length, r_ptr, remove_length))
 	end-=remove_length;
     }
   }
@@ -1422,7 +1422,7 @@ String *Item_func_soundex::val_str(String *str)
   char *to= (char *) tmp_value.ptr();
   char *from= (char *) res->ptr(), *end=from+res->length();
 
-  while (from != end && isspace(*from)) // Skip pre-space
+  while (from != end && !isalpha(*from)) // Skip pre-space
     from++; /* purecov: inspected */
   if (from == end)
     return &empty_string;		// No alpha characters.
@@ -1507,7 +1507,11 @@ void Item_func_elt::fix_length_and_dec()
 {
   max_length=0;
   decimals=0;
-  for (uint i=1 ; i < arg_count ; i++)
+#if MYSQL_VERSION_ID < 40100
+  for (uint i= 0; i < arg_count ; i++)
+#else
+  for (uint i= 1; i < arg_count ; i++)
+#endif
   {
     set_if_bigger(max_length,args[i]->max_length);
     set_if_bigger(decimals,args[i]->decimals);
@@ -1706,14 +1710,11 @@ inline String* alloc_buffer(String *res,String *str,String *tmp_value,
       str->length(length);
       return str;
     }
-    else
-    {
-      if (tmp_value->alloc(length))
-	return 0;
-      (void) tmp_value->copy(*res);
-      tmp_value->length(length);
-      return tmp_value;
-    }
+    if (tmp_value->alloc(length))
+      return 0;
+    (void) tmp_value->copy(*res);
+    tmp_value->length(length);
+    return tmp_value;
   }
   res->length(length);
   return res;
@@ -1806,7 +1807,7 @@ String *Item_func_rpad::val_str(String *str)
   const char *ptr_pad;
   int32 count= (int32) args[1]->val_int();
   String *res =args[0]->val_str(str);
-  String *rpad = args[2]->val_str(str);
+  String *rpad = args[2]->val_str(&rpad_str);
 
   if (!res || args[1]->null_value || !rpad || count < 0)
     goto err;
@@ -1866,7 +1867,7 @@ String *Item_func_lpad::val_str(String *str)
   const char *ptr_pad;
   ulong count= (long) args[1]->val_int();
   String *res= args[0]->val_str(str);
-  String *lpad= args[2]->val_str(str);
+  String *lpad= args[2]->val_str(&lpad_str);
 
   if (!res || args[1]->null_value || !lpad)
     goto err;
@@ -2043,6 +2044,10 @@ String* Item_func_export_set::val_str(String* str)
     null_value=1;
     return 0;
   }
+  /*
+    Arg count can only be 3, 4 or 5 here. This is guaranteed from the
+    grammar for EXPORT_SET()
+  */
   switch(arg_count) {
   case 5:
     num_set_values = (uint) args[4]->val_int();
@@ -2137,9 +2142,12 @@ String* Item_func_inet_ntoa::val_str(String* str)
 
     This function is very useful when you want to generate SQL statements
 
-    RETURN VALUES
+  NOTE
+    QUOTE(NULL) returns the string 'NULL' (4 letters, without quotes).
+
+  RETURN VALUES
     str		Quoted string
-    NULL	Argument to QUOTE() was NULL or out of memory.
+    NULL	Out of memory.
 */
 
 #define get_esc_bit(mask, num) (1 & (*((mask) + ((num) >> 3))) >> ((num) & 7))
@@ -2163,12 +2171,17 @@ String *Item_func_quote::val_str(String *str)
   String *arg= args[0]->val_str(str);
   uint arg_length, new_length;
   if (!arg)					// Null argument
-    goto null;
+  {
+    str->copy("NULL", 4);			// Return the string 'NULL'
+    null_value= 0;
+    return str;
+  }
+
   arg_length= arg->length();
   new_length= arg_length+2; /* for beginning and ending ' signs */
 
   for (from= (char*) arg->ptr(), end= from + arg_length; from < end; from++)
-    new_length+= get_esc_bit(escmask, *from);
+    new_length+= get_esc_bit(escmask, (uchar) *from);
 
   /*
     We have to use realloc() instead of alloc() as we want to keep the
@@ -2210,6 +2223,7 @@ String *Item_func_quote::val_str(String *str)
   }
   *to= '\'';
   str->length(new_length);
+  null_value= 0;
   return str;
 
 null:
