@@ -23,6 +23,11 @@
 #define mysql_unix_port mysql_inix_port1
 #define mysql_port mysql_port1
 
+extern "C"
+{
+  unsigned long max_allowed_packet, net_buffer_length;
+}
+
 #if defined (__WIN__)
 #include "../sql/mysqld.cpp"
 #else
@@ -315,7 +320,6 @@ extern "C"
 
 static my_bool  org_my_init_done;
 my_bool         server_inited;
-ulong		max_allowed_packet, net_buffer_length;
 char **		copy_arguments_ptr= 0; 
 
 int STDCALL mysql_server_init(int argc, char **argv, char **groups)
@@ -385,11 +389,6 @@ int STDCALL mysql_server_init(int argc, char **argv, char **groups)
 
   if (gethostname(glob_hostname,sizeof(glob_hostname)-4) < 0)
     strmov(glob_hostname,"mysql");
-#ifndef DBUG_OFF
-  strxmov(strend(server_version),MYSQL_SERVER_SUFFIX,"-debug",NullS);
-#else
-  strmov(strend(server_version),MYSQL_SERVER_SUFFIX);
-#endif
   load_defaults("my", (const char **) groups, argcp, argvp);
   defaults_argv=*argvp;
 
@@ -406,9 +405,8 @@ int STDCALL mysql_server_init(int argc, char **argv, char **groups)
 
   set_options();
   get_options(*argcp, *argvp);
+  set_server_version();
 
-  if (opt_log || opt_update_log || opt_slow_log || opt_bin_log)
-    strcat(server_version,"-log");
   DBUG_PRINT("info",("%s  Ver %s for %s on %s\n",my_progname,
 		     server_version, SYSTEM_TYPE,MACHINE_TYPE));
 
