@@ -46,6 +46,9 @@
 #define INADDR_NONE	-1
 #endif
 
+extern ulong net_buffer_length;
+extern ulong max_allowed_packet;
+
 static my_bool	mysql_client_init=0;
 uint		mysql_port=0;
 my_string	mysql_unix_port=0;
@@ -409,8 +412,8 @@ mysql_free_result(MYSQL_RES *result)
       DBUG_PRINT("warning",("Not all rows in set were read; Ignoring rows"));
       for (;;)
       {
-	uint pkt_len;
-	if ((pkt_len=(uint) net_safe_read(result->handle)) == packet_error)
+	ulong pkt_len;
+	if ((pkt_len=net_safe_read(result->handle)) == packet_error)
 	  break;
 	if (pkt_len == 1 && result->handle->net.read_pos[0] == 254)
 	  break;				/* End of data */
@@ -608,7 +611,8 @@ unpack_fields(MYSQL_DATA *data,MEM_ROOT *alloc,uint fields,
 static MYSQL_DATA *read_rows(MYSQL *mysql,MYSQL_FIELD *mysql_fields,
 			     uint fields)
 {
-  uint	field,pkt_len;
+  uint	field;
+  ulong pkt_len;
   ulong len;
   uchar *cp;
   char	*to;
@@ -617,7 +621,7 @@ static MYSQL_DATA *read_rows(MYSQL *mysql,MYSQL_FIELD *mysql_fields,
   NET *net = &mysql->net;
   DBUG_ENTER("read_rows");
 
-  if ((pkt_len=(uint) net_safe_read(mysql)) == packet_error)
+  if ((pkt_len= net_safe_read(mysql)) == packet_error)
     DBUG_RETURN(0);
   if (!(result=(MYSQL_DATA*) my_malloc(sizeof(MYSQL_DATA),
 				       MYF(MY_WME | MY_ZEROFILL))))
@@ -722,6 +726,15 @@ read_one_row(MYSQL *mysql,uint fields,MYSQL_ROW row, ulong *lengths)
   return 0;
 }
 
+static MYSQL_PARAMETERS mysql_internal_parameters=
+{&max_allowed_packet, &net_buffer_length};
+
+MYSQL_PARAMETERS *STDCALL mysql_get_parameters()
+{
+  return &mysql_internal_parameters;
+}
+
+
 /****************************************************************************
 ** Init MySQL structure or allocate one
 ****************************************************************************/
@@ -793,7 +806,7 @@ mysql_real_connect(MYSQL *mysql,const char *host, const char *user,
 		   uint port, const char *unix_socket,uint client_flag)
 {
   char		buff[100],charset_name_buff[16],*end,*host_info, *charset_name;
-  uint		pkt_length;
+  ulong		pkt_length;
   ulong		max_allowed_packet;
   NET		*net= &mysql->net;
   DBUG_ENTER("mysql_real_connect");
@@ -1141,7 +1154,7 @@ mysql_read_query_result(MYSQL *mysql)
   uchar *pos;
   ulong field_count;
   MYSQL_DATA *fields;
-  uint length;
+  ulong length;
   DBUG_ENTER("mysql_read_query_result");
 
   if ((length=net_safe_read(mysql)) == packet_error)
