@@ -31,12 +31,55 @@
 #include <m_ctype.h>				// For test_if_number
 #include <assert.h>
 
+#ifdef __NT__
+#include "message.h"
+#endif
+
 MYSQL_LOG mysql_log,mysql_update_log,mysql_slow_log,mysql_bin_log;
 extern I_List<i_string> binlog_do_db, binlog_ignore_db;
 
 static bool test_if_number(const char *str,
 			   long *res, bool allow_wildcards);
 
+#ifdef __NT__
+static int eventSource = 0;
+
+void setup_windows_event_source() 
+{
+  HKEY    hRegKey= NULL; 
+  DWORD   dwError= 0;
+  TCHAR   szPath[MAX_PATH];
+  DWORD dwTypes;
+    
+  if (eventSource)               // Ensure that we are only called once
+    return;
+  eventSource= 1;
+
+  // Create the event source registry key
+  dwError= RegCreateKey(HKEY_LOCAL_MACHINE, 
+                          "SYSTEM\\CurrentControlSet\\Services\\EventLog\\Application\\MySQL", 
+                          &hRegKey);
+
+  /* Name of the PE module that contains the message resource */
+  GetModuleFileName(NULL, szPath, MAX_PATH);
+
+  /* Register EventMessageFile */
+  dwError = RegSetValueEx(hRegKey, "EventMessageFile", 0, REG_EXPAND_SZ, 
+                          (PBYTE) szPath, strlen(szPath)+1);
+    
+
+  /* Register supported event types */
+  dwTypes= (EVENTLOG_ERROR_TYPE | EVENTLOG_WARNING_TYPE |
+            EVENTLOG_INFORMATION_TYPE);
+  dwError= RegSetValueEx(hRegKey, "TypesSupported", 0, REG_DWORD,
+                         (LPBYTE) &dwTypes, sizeof dwTypes);
+
+  RegCloseKey(hRegKey);
+}
+
+#endif /* __NT__ */
+
+
 /****************************************************************************
 ** Find a uniq filename for 'filename.#'.
 ** Set # to a number as low as possible
@@ -202,7 +245,7 @@ bool MYSQL_LOG::open(const char *log_name, enum_log_type log_type_arg,
     open_flags |= O_RDWR;
   else
     open_flags |= O_WRONLY;
-  
+
   db[0]=0;
   open_count++;
   if ((file=my_open(log_file_name,open_flags,
@@ -215,12 +258,15 @@ bool MYSQL_LOG::open(const char *log_name, enum_log_type log_type_arg,
   case LOG_NORMAL:
   {
     char *end;
+    int len=my_snprintf(buff, sizeof(buff),
 #ifdef __NT__
-    sprintf(buff, "%s, Version: %s, started with:\nTCP Port: %d, Named Pipe: %s\n", my_progname, server_version, mysql_port, mysql_unix_port);
+                        "%s, Version: %s, started with:\nTCP Port: %d, Named Pipe: %s\n",
 #else
-    sprintf(buff, "%s, Version: %s, started with:\nTcp port: %d  Unix socket: %s\n", my_progname,server_version,mysql_port,mysql_unix_port);
+                        "%s, Version: %s, started with:\nTcp port: %d  Unix socket: %s\n",
 #endif
-    end=strmov(strend(buff),"Time                 Id Command    Argument\n");
+                        my_progname, server_version, mysql_port, mysql_unix_port);
+    end=strnmov(buff+len,"Time                 Id Command    Argument\n",
+                sizeof(buff)-len);
     if (my_b_write(&log_file, (byte*) buff,(uint) (end-buff)) ||
 	flush_io_cache(&log_file))
       goto err;
@@ -228,18 +274,21 @@ bool MYSQL_LOG::open(const char *log_name, enum_log_type log_type_arg,
   }
   case LOG_NEW:
   {
+    uint len;
     time_t skr=time(NULL);
     struct tm tm_tmp;
+
     localtime_r(&skr,&tm_tmp);
-    sprintf(buff,"# %s, Version: %s at %02d%02d%02d %2d:%02d:%02d\n",
-	    my_progname,server_version,
-	    tm_tmp.tm_year % 100,
-	    tm_tmp.tm_mon+1,
-	    tm_tmp.tm_mday,
-	    tm_tmp.tm_hour,
-	    tm_tmp.tm_min,
-	    tm_tmp.tm_sec);
-    if (my_b_write(&log_file, (byte*) buff,(uint) strlen(buff)) ||
+    len= my_snprintf(buff,sizeof(buff),
+		     "# %s, Version: %s at %02d%02d%02d %2d:%02d:%02d\n",
+		     my_progname,server_version,
+		     tm_tmp.tm_year % 100,
+		     tm_tmp.tm_mon+1,
+		     tm_tmp.tm_mday,
+		     tm_tmp.tm_hour,
+		     tm_tmp.tm_min,
+		     tm_tmp.tm_sec);
+    if (my_b_write(&log_file, (byte*) buff, len) ||
 	flush_io_cache(&log_file))
       goto err;
     break;
@@ -254,7 +303,7 @@ bool MYSQL_LOG::open(const char *log_name, enum_log_type log_type_arg,
       index_file_name_arg= name;	// Use same basename for index file
       opt= MY_UNPACK_FILENAME | MY_REPLACE_EXT;
     }
-  
+
     if (!my_b_filelength(&log_file))
     {
       /*
@@ -843,7 +892,7 @@ void MYSQL_LOG::new_file(bool need_lock)
         We log the whole file name for log file as the user may decide
         to change base names at some point.
       */
-      THD* thd = current_thd;
+      THD* thd = current_thd; /* may be 0 if we are reacting to SIGHUP */
       Rotate_log_event r(thd,new_name+dirname_length(new_name));
       r.set_log_pos(this);
       
@@ -852,7 +901,7 @@ void MYSQL_LOG::new_file(bool need_lock)
         the slave running with log-bin, we set the flag on rotate
         event to prevent infinite log rotation loop
       */
-      if (thd->slave_thread)
+      if (thd && thd->slave_thread)
         r.flags|= LOG_EVENT_FORCED_ROTATE_F;
       r.write(&log_file);
       bytes_written += r.get_event_len();
@@ -870,7 +919,7 @@ void MYSQL_LOG::new_file(bool need_lock)
   close(LOG_CLOSE_TO_BE_OPENED);
 
   /* 
-     Note that at this point, log_type == LOG_CLOSED (important for is_open()).
+     Note that at this point, log_type != LOG_CLOSED (important for is_open()).
   */
 
   open(old_name, save_log_type, new_name_ptr, index_file_name, io_cache_type,
@@ -1110,8 +1159,6 @@ bool MYSQL_LOG::write(Log_event* event_info)
 	Intvar_log_event e(thd,(uchar) LAST_INSERT_ID_EVENT,
 			   thd->current_insert_id);
 	e.set_log_pos(this);
-	if (thd->server_id)
-	  e.server_id = thd->server_id;
 	if (e.write(file))
 	  goto err;
       }
@@ -1119,8 +1166,6 @@ bool MYSQL_LOG::write(Log_event* event_info)
       {
 	Intvar_log_event e(thd,(uchar) INSERT_ID_EVENT,thd->last_insert_id);
 	e.set_log_pos(this);
-	if (thd->server_id)
-	  e.server_id = thd->server_id;
 	if (e.write(file))
 	  goto err;
       }
@@ -1167,12 +1212,24 @@ bool MYSQL_LOG::write(Log_event* event_info)
 
     /* Write log events to reset the 'run environment' of the SQL command */
 
-    if (thd && thd->options & OPTION_NO_FOREIGN_KEY_CHECKS)
+    if (thd)
     {
-      Query_log_event e(thd, "SET FOREIGN_KEY_CHECKS=1", 24, 0);
-      e.set_log_pos(this);
-      if (e.write(file))
-	goto err;
+      if (thd->options & OPTION_NO_FOREIGN_KEY_CHECKS)
+      {
+        Query_log_event e(thd, "SET FOREIGN_KEY_CHECKS=1", 24, 0);
+        e.set_log_pos(this);
+        if (e.write(file))
+          goto err;
+      }
+#if MYSQL_VERSION_ID < 40100
+      if (thd->variables.convert_set)
+      {
+	Query_log_event e(thd, "SET CHARACTER SET DEFAULT", 25, 0);
+	e.set_log_pos(this);
+	if (e.write(file))
+	  goto err;
+      }
+#endif
     }
 
     /*
@@ -1197,7 +1254,8 @@ bool MYSQL_LOG::write(Log_event* event_info)
       if (flush_io_cache(file))
 	goto err;
  
-      if (opt_using_transactions && !my_b_tell(&thd->transaction.trans_log))
+      if (opt_using_transactions &&
+          !(thd->options & (OPTION_NOT_AUTOCOMMIT | OPTION_BEGIN)))
       {
         /*
           LOAD DATA INFILE in AUTOCOMMIT=1 mode writes to the binlog
@@ -1312,12 +1370,20 @@ bool MYSQL_LOG::write(THD *thd, IO_CACHE *cache, bool commit_or_rollback)
     */
     {
       Query_log_event qinfo(thd, "BEGIN", 5, TRUE);
+      /*
+        Imagine this is rollback due to net timeout, after all statements of
+        the transaction succeeded. Then we want a zero-error code in BEGIN.
+        In other words, if there was a really serious error code it's already
+        in the statement's events.
+        This is safer than thd->clear_error() against kills at shutdown.
+      */
+      qinfo.error_code= 0;
       /*
         Now this Query_log_event has artificial log_pos 0. It must be adjusted
         to reflect the real position in the log. Not doing it would confuse the
-        slave: it would prevent this one from knowing where he is in the master's
-        binlog, which would result in wrong positions being shown to the user,
-        MASTER_POS_WAIT undue waiting etc.
+	slave: it would prevent this one from knowing where he is in the
+	master's binlog, which would result in wrong positions being shown to
+	the user, MASTER_POS_WAIT undue waiting etc.
       */
       qinfo.set_log_pos(this);
       if (qinfo.write(&log_file))
@@ -1345,6 +1411,7 @@ bool MYSQL_LOG::write(THD *thd, IO_CACHE *cache, bool commit_or_rollback)
                             commit_or_rollback ? "COMMIT" : "ROLLBACK",
                             commit_or_rollback ? 6        : 8, 
                             TRUE);
+      qinfo.error_code= 0;
       qinfo.set_log_pos(this);
       if (qinfo.write(&log_file) || flush_io_cache(&log_file))
 	goto err;
@@ -1531,18 +1598,13 @@ bool MYSQL_LOG::write(THD *thd,const char *query, uint query_length,
 
   NOTES
     One must have a lock on LOCK_log before calling this function.
-    This lock will be freed before return!
-
-    The reason for the above is that for enter_cond() / exit_cond() to
-    work the mutex must be got before enter_cond() but releases before
-    exit_cond().
-    If you don't do it this way, you will get a deadlock in THD::awake()
+    This lock will be freed before return! That's required by
+    THD::enter_cond() (see NOTES in sql_class.h).
 */
 
 
 void MYSQL_LOG:: wait_for_update(THD* thd, bool master_or_slave)
 {
-  safe_mutex_assert_owner(&LOCK_log);
   const char* old_msg = thd->enter_cond(&update_cond, &LOCK_log,
                                         master_or_slave ?
                                         "Has read all relay log; waiting for \
@@ -1550,7 +1612,6 @@ the I/O slave thread to update it" :
                                         "Has sent all binlog to slave; \
 waiting for binlog to be updated"); 
   pthread_cond_wait(&update_cond, &LOCK_log);
-  pthread_mutex_unlock(&LOCK_log);		// See NOTES
   thd->exit_cond(old_msg);
 }
 
@@ -1681,39 +1742,37 @@ static bool test_if_number(register const char *str,
 } /* test_if_number */
 
 
-void sql_print_error(const char *format,...)
+void print_buffer_to_file(enum loglevel level, const char *buffer)
 {
-  va_list args;
   time_t skr;
   struct tm tm_tmp;
   struct tm *start;
-  va_start(args,format);
-  DBUG_ENTER("sql_print_error");
+  DBUG_ENTER("print_buffer_to_file");
+  DBUG_PRINT("enter",("buffer: %s", buffer));
 
   VOID(pthread_mutex_lock(&LOCK_error_log));
-#ifndef DBUG_OFF
-  {
-    char buff[1024];
-    my_vsnprintf(buff,sizeof(buff)-1,format,args);
-    DBUG_PRINT("error",("%s",buff));
-    va_end(args);
-    va_start(args,format);
-  }
-#endif
+
   skr=time(NULL);
-  localtime_r(&skr,&tm_tmp);
+  localtime_r(&skr, &tm_tmp);
   start=&tm_tmp;
-  fprintf(stderr,"%02d%02d%02d %2d:%02d:%02d  ",
-	  start->tm_year % 100,
-	  start->tm_mon+1,
-	  start->tm_mday,
-	  start->tm_hour,
-	  start->tm_min,
-	  start->tm_sec);
-  (void) vfprintf(stderr,format,args);
-  (void) fputc('\n',stderr);
+#if MYSQL_VERSION_ID > 40100
+  fprintf(stderr, "%02d%02d%02d %2d:%02d:%02d  [%s] %s\n",
+#else
+  fprintf(stderr, "%02d%02d%02d %2d:%02d:%02d %s\n",
+#endif
+          start->tm_year % 100,
+          start->tm_mon+1,
+          start->tm_mday,
+          start->tm_hour,
+          start->tm_min,
+          start->tm_sec,
+#if MYSQL_VERSION_ID > 40100
+          (level == ERROR_LEVEL ? "ERROR" : level == WARNING_LEVEL ?
+           "WARNING" : "INFORMATION"),
+#endif
+          buffer);
+
   fflush(stderr);
-  va_end(args);
 
   VOID(pthread_mutex_unlock(&LOCK_error_log));
   DBUG_VOID_RETURN;
@@ -1729,6 +1788,7 @@ void sql_perror(const char *message)
 #endif
 }
 
+
 bool flush_error_log()
 {
   bool result=0;
@@ -1737,6 +1797,7 @@ bool flush_error_log()
     char err_renamed[FN_REFLEN], *end;
     end= strmake(err_renamed,log_error_file,FN_REFLEN-4);
     strmov(end, "-old");
+    VOID(pthread_mutex_lock(&LOCK_error_log));
 #ifdef __WIN__
     char err_temp[FN_REFLEN+4];
     /*
@@ -1744,7 +1805,7 @@ bool flush_error_log()
      the current error file.
     */
     strmov(strmov(err_temp, err_renamed),"-tmp");
-    (void) my_delete(err_temp, MYF(0)); 
+    (void) my_delete(err_temp, MYF(0));
     if (freopen(err_temp,"a+",stdout))
     {
       freopen(err_temp,"a+",stderr);
@@ -1757,20 +1818,145 @@ bool flush_error_log()
       if ((fd = my_open(err_temp, O_RDONLY, MYF(0))) >= 0)
       {
         while ((bytes = (int) my_read(fd, (byte*) buf, IO_SIZE, MYF(0))) > 0)
-             my_fwrite(stderr, (byte*) buf, (uint) strlen(buf),MYF(0));
+             my_fwrite(stderr, (byte*) buf, bytes, MYF(0));
         my_close(fd, MYF(0));
       }
-      (void) my_delete(err_temp, MYF(0)); 
+      (void) my_delete(err_temp, MYF(0));
     }
     else
      result= 1;
 #else
-   my_rename(log_error_file,err_renamed,MYF(0));
-   if (freopen(log_error_file,"a+",stdout))
-     freopen(log_error_file,"a+",stderr);
-   else
-     result= 1;
+    my_rename(log_error_file,err_renamed,MYF(0));
+    if (freopen(log_error_file,"a+",stdout))
+      freopen(log_error_file,"a+",stderr);
+    else
+      result= 1;
 #endif
+    VOID(pthread_mutex_unlock(&LOCK_error_log));
   }
    return result;
 }
+
+
+#ifdef __NT__
+void print_buffer_to_nt_eventlog(enum loglevel level, char *buff,
+                                 uint length, int buffLen)
+{
+  HANDLE event;
+  char   *buffptr;
+  LPCSTR *buffmsgptr;
+  DBUG_ENTER("print_buffer_to_nt_eventlog");
+
+  buffptr= buff;
+  if (length > (uint)(buffLen-4))
+  {
+    char *newBuff= new char[length + 4];
+    strcpy(newBuff, buff);
+    buffptr= newBuff;
+  }
+  strmov(buffptr+length, "\r\n\r\n");
+  buffmsgptr= (LPCSTR*) &buffptr;               // Keep windows happy
+
+  setup_windows_event_source();
+  if ((event= RegisterEventSource(NULL,"MySQL")))
+  {
+    switch (level) {
+      case ERROR_LEVEL:
+        ReportEvent(event, EVENTLOG_ERROR_TYPE, 0, MSG_DEFAULT, NULL, 1, 0,
+                    buffmsgptr, NULL);
+        break;
+      case WARNING_LEVEL:
+        ReportEvent(event, EVENTLOG_WARNING_TYPE, 0, MSG_DEFAULT, NULL, 1, 0,
+                    buffmsgptr, NULL);
+        break;
+      case INFORMATION_LEVEL:
+        ReportEvent(event, EVENTLOG_INFORMATION_TYPE, 0, MSG_DEFAULT, NULL, 1,
+                    0, buffmsgptr, NULL);
+        break;
+    }
+    DeregisterEventSource(event);
+  }
+
+  /* if we created a string buffer, then delete it */
+  if (buffptr != buff)
+    delete[] buffptr;
+
+  DBUG_VOID_RETURN;
+}
+#endif /* __NT__ */
+
+
+/*
+  Prints a printf style message to the error log and, under NT, to the
+  Windows event log.
+
+  SYNOPSIS
+    vprint_msg_to_log()
+    event_type             Type of event to write (Error, Warning, or Info)
+    format                 Printf style format of message
+    args                   va_list list of arguments for the message    
+
+  NOTE
+
+  IMPLEMENTATION
+    This function prints the message into a buffer and then sends that buffer
+    to other functions to write that message to other logging sources.
+
+  RETURN VALUES
+    void
+*/
+
+void vprint_msg_to_log(enum loglevel level, const char *format, va_list args)
+{
+  char   buff[1024];
+  uint length;
+  DBUG_ENTER("vprint_msg_to_log");
+
+  length= my_vsnprintf(buff, sizeof(buff)-5, format, args);
+  print_buffer_to_file(level, buff);
+
+#ifdef __NT__
+  print_buffer_to_nt_eventlog(level, buff, length, sizeof(buff));
+#endif
+
+  DBUG_VOID_RETURN;
+}
+
+
+void sql_print_error(const char *format, ...) 
+{
+  va_list args;
+  DBUG_ENTER("sql_print_error");
+
+  va_start(args, format);
+  vprint_msg_to_log(ERROR_LEVEL, format, args);
+  va_end(args);
+
+  DBUG_VOID_RETURN;
+}
+
+
+void sql_print_warning(const char *format, ...) 
+{
+  va_list args;
+  DBUG_ENTER("sql_print_warning");
+
+  va_start(args, format);
+  vprint_msg_to_log(WARNING_LEVEL, format, args);
+  va_end(args);
+
+  DBUG_VOID_RETURN;
+}
+
+
+void sql_print_information(const char *format, ...) 
+{
+  va_list args;
+  DBUG_ENTER("sql_print_information");
+
+  va_start(args, format);
+  vprint_msg_to_log(INFORMATION_LEVEL, format, args);
+  va_end(args);
+
+  DBUG_VOID_RETURN;
+}
