@@ -70,7 +70,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   outparam->db_stat = db_stat;
   error=1;
 
-  init_sql_alloc(&outparam->mem_root,1024,0);
+  init_sql_alloc(&outparam->mem_root, TABLE_ALLOC_BLOCK_SIZE, 0);
   MEM_ROOT *old_root=my_pthread_getspecific_ptr(MEM_ROOT*,THR_MALLOC);
   my_pthread_setspecific_ptr(THR_MALLOC,&outparam->mem_root);
 
@@ -430,14 +430,17 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
       if (primary_key >= MAX_KEY && (keyinfo->flags & HA_NOSAME))
       {
 	/*
-	  If the UNIQUE key don't have NULL columns, declare this as
-	  a primary key.
+	  If the UNIQUE key doesn't have NULL columns and is not a part key
+	  declare this as a primary key.
 	*/
 	primary_key=key;
 	for (i=0 ; i < keyinfo->key_parts ;i++)
 	{
-	  if (!key_part[i].fieldnr ||
-	      outparam->field[key_part[i].fieldnr-1]->null_ptr)
+	  uint fieldnr= key_part[i].fieldnr;
+	  if (!fieldnr ||
+	      outparam->field[fieldnr-1]->null_ptr ||
+	      outparam->field[fieldnr-1]->key_length() !=
+	      key_part[i].length)
 	  {
 	    primary_key=MAX_KEY;		// Can't be used
 	    break;
@@ -476,6 +479,12 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	    keyinfo->extra_length+=HA_KEY_BLOB_LENGTH;
 	    key_part->store_length+=HA_KEY_BLOB_LENGTH;
 	    keyinfo->key_length+= HA_KEY_BLOB_LENGTH;
+	    /*
+	      Mark that there may be many matching values for one key
+	      combination ('a', 'a ', 'a  '...)
+	    */
+	    if (!(field->flags & BINARY_FLAG))
+	      keyinfo->flags|= HA_END_SPACE_KEY;
 	  }
 	  if (i == 0 && key != primary_key)
 	    field->flags |=
@@ -489,7 +498,8 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	      field->type() != FIELD_TYPE_BLOB)
 	  {
 	    if (field->key_type() != HA_KEYTYPE_TEXT ||
-		(!(ha_option & HA_KEY_READ_WRONG_STR) &&
+		((!(ha_option & HA_KEY_READ_WRONG_STR) ||
+		  field->flags & BINARY_FLAG) &&
 		 !(keyinfo->flags & HA_FULLTEXT)))
 	      field->part_of_key|= ((key_map) 1 << key);
 	    if ((field->key_type() != HA_KEYTYPE_TEXT ||
@@ -513,7 +523,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	  }
 	  if (field->key_length() != key_part->length)
 	  {
-	    key_part->key_part_flag|= HA_PART_KEY;
+	    key_part->key_part_flag|= HA_PART_KEY_SEG;
 	    if (field->type() != FIELD_TYPE_BLOB)
 	    {					// Create a new field
 	      field=key_part->field=field->new_field(&outparam->mem_root,
@@ -527,7 +537,7 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
 	    as we need to test for NULL = NULL.
 	  */
 	  if (field->real_maybe_null())
-	    key_part->key_part_flag|= HA_PART_KEY;
+	    key_part->key_part_flag|= HA_PART_KEY_SEG;
 	}
 	else
 	{					// Error: shorten key
@@ -608,8 +618,9 @@ int openfrm(const char *name, const char *alias, uint db_stat, uint prgflag,
   if (db_stat)
   {
     int err;
+    unpack_filename(index_file,index_file);
     if ((err=(outparam->file->
-	      ha_open(unpack_filename(index_file,index_file),
+	      ha_open(index_file,
 		      (db_stat & HA_READ_ONLY ? O_RDONLY : O_RDWR),
 		      (db_stat & HA_OPEN_TEMPORARY ? HA_OPEN_TMP_TABLE :
 		       ((db_stat & HA_WAIT_IF_LOCKED) ||
@@ -982,8 +993,23 @@ ulong next_io_size(register ulong pos)
 
 void append_unescaped(String *res,const char *pos)
 {
-  for (; *pos ; pos++)
+#ifdef USE_MB
+  const char *end= pos + strlen(pos);
+#endif
+
+  for (; *pos ; )
   {
+#if defined(USE_MB) && MYSQL_VERSION_ID < 40100
+    uint mblen;
+    if (use_mb(default_charset_info) &&
+        (mblen= my_ismbchar(default_charset_info, pos, end)))
+    {
+      res->append(pos, mblen);
+      pos+= mblen;
+      continue;
+    }
+#endif
+
     switch (*pos) {
     case 0:				/* Must be escaped for 'mysql' */
       res->append('\\');
@@ -1009,6 +1035,7 @@ void append_unescaped(String *res,const char *pos)
       res->append(*pos);
       break;
     }
+    pos++;
   }
 }
 
@@ -1029,6 +1056,11 @@ File create_frm(register my_string name, uint reclength, uchar *fileinfo,
   if (create_info->min_rows > ~(ulong) 0)
     create_info->min_rows= ~(ulong) 0;
 #endif
+  /*
+    Ensure that raid_chunks can't be larger than 255, as this would cause
+    problems with drop database
+  */
+  set_if_smaller(create_info->raid_chunks, 255);
 
   if ((file=my_create(name,CREATE_MODE,O_RDWR | O_TRUNC,MYF(MY_WME))) >= 0)
   {
@@ -1100,13 +1132,12 @@ rename_file_ext(const char * from,const char * to,const char * ext)
 char *get_field(MEM_ROOT *mem, TABLE *table, uint fieldnr)
 {
   Field *field=table->field[fieldnr];
-  char buff[MAX_FIELD_WIDTH];
+  char buff[MAX_FIELD_WIDTH], *to;
   String str(buff,sizeof(buff));
   field->val_str(&str,&str);
   uint length=str.length();
-  if (!length)
+  if (!length || !(to= (char*) alloc_root(mem,length+1)))
     return NullS;
-  char *to= (char*) alloc_root(mem,length+1);
   memcpy(to,str.ptr(),(uint) length);
   to[length]=0;
   return to;
@@ -1130,7 +1161,8 @@ char *get_field(MEM_ROOT *mem, TABLE *table, uint fieldnr)
 
 bool check_db_name(char *name)
 {
-   char *start=name;
+  char *start=name;
+  bool last_char_is_space= FALSE;
 
   if (lower_case_table_names)
     casedn_str(name);
@@ -1138,6 +1170,7 @@ bool check_db_name(char *name)
   while (*name)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, name+MBMAXLEN);
@@ -1147,19 +1180,22 @@ bool check_db_name(char *name)
         continue;
       }
     }
+#else
+    last_char_is_space= *name==' ';
 #endif
     if (*name == '/' || *name == '\\' || *name == FN_LIBCHAR ||
 	*name == FN_EXTCHAR)
       return 1;
     name++;
   }
-  return (uint) (name - start) > NAME_LEN;
+  return last_char_is_space || (uint) (name - start) > NAME_LEN;
 }
 
 
 /*
   Allow anything as a table name, as long as it doesn't contain an
   a '/', or a '.' character
+  or ' ' at the end
   returns 1 on error
 */
 
@@ -1169,10 +1205,17 @@ bool check_table_name(const char *name, uint length)
   const char *end= name+length;
   if (!length || length > NAME_LEN)
     return 1;
+#if defined(USE_MB) && defined(USE_MB_IDENT)
+  bool last_char_is_space= FALSE;
+#else
+  if (name[length-1]==' ')
+    return 1;
+#endif
 
   while (name != end)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, end);
@@ -1187,16 +1230,23 @@ bool check_table_name(const char *name, uint length)
       return 1;
     name++;
   }
+#if defined(USE_MB) && defined(USE_MB_IDENT)
+  return last_char_is_space;
+#else
   return 0;
+#endif
 }
 
+
 bool check_column_name(const char *name)
 {
   const char *start= name;
-
+  bool last_char_is_space= FALSE;
+  
   while (*name)
   {
 #if defined(USE_MB) && defined(USE_MB_IDENT)
+    last_char_is_space= my_isspace(default_charset_info, *name);
     if (use_mb(default_charset_info))
     {
       int len=my_ismbchar(default_charset_info, name, name+MBMAXLEN);
@@ -1206,13 +1256,15 @@ bool check_column_name(const char *name)
         continue;
       }
     }
+#else
+    last_char_is_space= *name==' ';
 #endif
     if (*name == NAMES_SEP_CHAR)
       return 1;
     name++;
   }
   /* Error if empty or too long column name */
-  return (name == start || (uint) (name - start) > NAME_LEN);
+  return last_char_is_space || (name == start || (uint) (name - start) > NAME_LEN);
 }
 
 /*
