@@ -17,10 +17,13 @@
 #include <my_global.h>
 #include <m_string.h>
 #include <stdlib.h>
-#include <my_getopt.h>
 #include <assert.h>
 #include <my_sys.h>
 #include <mysys_err.h>
+#include <my_getopt.h>
+
+static void default_reporter(enum loglevel level, const char *format, ...);
+my_error_reporter my_getopt_error_reporter= &default_reporter;
 
 static int findopt(char *optpat, uint length,
 		   const struct my_option **opt_res,
@@ -56,6 +59,14 @@ char *disabled_my_option= (char*) "0";
 
 my_bool my_getopt_print_errors= 1;
 
+static void default_reporter(enum loglevel level __attribute__((unused)),
+                             const char *format, ...)
+{
+  va_list args;
+  va_start(args, format);
+  vfprintf(stderr, format, args);
+  va_end(args);
+}
 
 /* 
   function: handle_options
@@ -68,10 +79,8 @@ my_bool my_getopt_print_errors= 1;
 */
 
 int handle_options(int *argc, char ***argv, 
-		   const struct my_option *longopts, 
-		   my_bool (*get_one_option)(int,
-					     const struct my_option *,
-					     char *))
+		   const struct my_option *longopts,
+                   my_get_one_option get_one_option)
 {
   uint opt_found, argvpos= 0, length, i;
   my_bool end_of_options= 0, must_be_var, set_maximum_value, special_used,
@@ -109,8 +118,9 @@ int handle_options(int *argc, char ***argv,
 	    if (!*++pos)
 	    {
 	      if (my_getopt_print_errors)
-		fprintf(stderr, "%s: Option '-O' requires an argument\n",
-			progname);
+                my_getopt_error_reporter(ERROR_LEVEL,
+                                         "%s: Option '-O' requires an argument\n",
+                                         progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
 	    cur_arg= *pos;
@@ -126,9 +136,9 @@ int handle_options(int *argc, char ***argv,
 	    if (!*cur_arg)
 	    {
 	      if (my_getopt_print_errors)
-		fprintf(stderr,
-			"%s: Option '--set-variable' requires an argument\n",
-			progname);
+                my_getopt_error_reporter(ERROR_LEVEL,
+                                         "%s: Option '--set-variable' requires an argument\n",
+                                         progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
 	  }
@@ -140,9 +150,9 @@ int handle_options(int *argc, char ***argv,
 	    if (!*++pos)
 	    {
 	      if (my_getopt_print_errors)
-		fprintf(stderr,
-			"%s: Option '--set-variable' requires an argument\n",
-			progname);
+                my_getopt_error_reporter(ERROR_LEVEL,
+                                         "%s: Option '--set-variable' requires an argument\n",
+                                         progname);
 	      return EXIT_ARGUMENT_REQUIRED;
 	    }
 	    cur_arg= *pos;
@@ -201,10 +211,11 @@ int handle_options(int *argc, char ***argv,
 		  if (opt_found > 1)
 		  {
 		    if (my_getopt_print_errors)
-		      fprintf(stderr,
-			      "%s: ambiguous option '--%s-%s' (--%s-%s)\n",
-			      progname, special_opt_prefix[i], cur_arg,
-			      special_opt_prefix[i], prev_found);
+                      my_getopt_error_reporter(ERROR_LEVEL,
+                                               "%s: ambiguous option '--%s-%s' (--%s-%s)\n",
+                                               progname, special_opt_prefix[i],
+                                               cur_arg, special_opt_prefix[i],
+                                               prev_found);
 		    return EXIT_AMBIGUOUS_OPTION;
 		  }
 		  switch (i) {
@@ -236,18 +247,20 @@ int handle_options(int *argc, char ***argv,
 	    if (must_be_var)
 	    {
 	      if (my_getopt_print_errors)
-		fprintf(stderr,
-			"%s: %s: unknown variable '%s'\n", progname,
-			option_is_loose ? "WARNING" : "ERROR", cur_arg);
+                my_getopt_error_reporter(option_is_loose ? 
+                                           WARNING_LEVEL : ERROR_LEVEL,
+                                         "%s: unknown variable '%s'\n",
+                                         progname, cur_arg);
 	      if (!option_is_loose)
 		return EXIT_UNKNOWN_VARIABLE;
 	    }
 	    else
 	    {
 	      if (my_getopt_print_errors)
-		fprintf(stderr,
-			"%s: %s: unknown option '--%s'\n", progname,
-			option_is_loose ? "WARNING" : "ERROR", cur_arg);
+                my_getopt_error_reporter(option_is_loose ? 
+                                           WARNING_LEVEL : ERROR_LEVEL,
+                                         "%s: unknown option '--%s'\n", 
+                                         progname, cur_arg);
 	      if (!option_is_loose)
 		return EXIT_UNKNOWN_OPTION;
 	    }
@@ -263,23 +276,27 @@ int handle_options(int *argc, char ***argv,
 	  if (must_be_var)
 	  {
 	    if (my_getopt_print_errors)
-	      fprintf(stderr, "%s: variable prefix '%s' is not unique\n",
-		      progname, cur_arg);
+              my_getopt_error_reporter(ERROR_LEVEL,
+                                       "%s: variable prefix '%s' is not unique\n",
+                                       progname, cur_arg);
 	    return EXIT_VAR_PREFIX_NOT_UNIQUE;
 	  }
 	  else
 	  {
 	    if (my_getopt_print_errors)
-	      fprintf(stderr, "%s: ambiguous option '--%s' (%s, %s)\n",
-		      progname, cur_arg, prev_found, optp->name);
+              my_getopt_error_reporter(ERROR_LEVEL,
+                                       "%s: ambiguous option '--%s' (%s, %s)\n",
+                                       progname, cur_arg, prev_found, 
+                                       optp->name);
 	    return EXIT_AMBIGUOUS_OPTION;
 	  }
 	}
 	if (must_be_var && optp->var_type == GET_NO_ARG)
 	{
 	  if (my_getopt_print_errors)
-	    fprintf(stderr, "%s: option '%s' cannot take an argument\n",
-		    progname, optp->name);
+            my_getopt_error_reporter(ERROR_LEVEL, 
+                                     "%s: option '%s' cannot take an argument\n",
+                                     progname, optp->name);
 	  return EXIT_NO_ARGUMENT_ALLOWED;
 	}
 	if (optp->arg_type == NO_ARG)
@@ -287,8 +304,9 @@ int handle_options(int *argc, char ***argv,
 	  if (optend && optp->var_type != GET_BOOL)
 	  {
 	    if (my_getopt_print_errors)
-	      fprintf(stderr, "%s: option '--%s' cannot take an argument\n",
-		      progname, optp->name);
+              my_getopt_error_reporter(ERROR_LEVEL,
+                                       "%s: option '--%s' cannot take an argument\n",
+                                       progname, optp->name);
 	    return EXIT_NO_ARGUMENT_ALLOWED;
 	  }
 	  if (optp->var_type == GET_BOOL)
@@ -325,8 +343,9 @@ int handle_options(int *argc, char ***argv,
 	  if (!*++pos)
 	  {
 	    if (my_getopt_print_errors)
-	      fprintf(stderr, "%s: option '--%s' requires an argument\n",
-		      progname, optp->name);
+              my_getopt_error_reporter(ERROR_LEVEL,
+                                       "%s: option '--%s' requires an argument\n",
+                                       progname, optp->name);
 	    return EXIT_ARGUMENT_REQUIRED;
 	  }
 	  argument= *pos;
@@ -375,9 +394,9 @@ int handle_options(int *argc, char ***argv,
 		  if (!pos[1])
 		  {
                     if (my_getopt_print_errors)
-                      fprintf(stderr,
-                              "%s: option '-%c' requires an argument\n",
-                              progname, optp->id);
+                      my_getopt_error_reporter(ERROR_LEVEL,
+                                               "%s: option '-%c' requires an argument\n",
+                                               progname, optp->id);
                     return EXIT_ARGUMENT_REQUIRED;
 		  }
 		  argument= *++pos;
@@ -387,9 +406,9 @@ int handle_options(int *argc, char ***argv,
 	      }
 	      if ((error= setval(optp, argument, set_maximum_value)))
 	      {
-		fprintf(stderr,
-			"%s: Error while setting value '%s' to '%s'\n",
-			progname, argument, optp->name);
+                my_getopt_error_reporter(ERROR_LEVEL,
+                                         "%s: Error while setting value '%s' to '%s'\n",
+                                         progname, argument, optp->name);
 		return error;
 	      }
 	      get_one_option(optp->id, optp, argument);
@@ -399,8 +418,9 @@ int handle_options(int *argc, char ***argv,
 	  if (!opt_found)
 	  {
 	    if (my_getopt_print_errors)
-	      fprintf(stderr,
-		      "%s: unknown option '-%c'\n", progname, *optend);
+              my_getopt_error_reporter(ERROR_LEVEL,
+                                       "%s: unknown option '-%c'\n", 
+                                       progname, *optend);
 	    return EXIT_UNKNOWN_OPTION;
 	  }
 	}
@@ -409,9 +429,9 @@ int handle_options(int *argc, char ***argv,
       }
       if ((error= setval(optp, argument, set_maximum_value)))
       {
-	fprintf(stderr,
-		"%s: Error while setting value '%s' to '%s'\n",
-		progname, argument, optp->name);
+        my_getopt_error_reporter(ERROR_LEVEL,
+                                 "%s: Error while setting value '%s' to '%s'\n",
+                                 progname, argument, optp->name);
 	return error;
       }
       get_one_option(optp->id, optp, argument);
@@ -690,6 +710,8 @@ static void init_variables(const struct my_option *options)
   Print help for all options and variables.
 */
 
+#include <help_start.h>
+
 void my_print_help(const struct my_option *options)
 {
   uint col, name_space= 22, comment_space= 57;
@@ -815,3 +837,5 @@ void my_print_variables(const struct my_option *options)
     }
   }
 }
+
+#include <help_end.h>
