@@ -189,6 +189,10 @@ public:
   {
     return pack(to,from,max_length);
   }
+  virtual const char *unpack_key(char* to, const char *from, uint max_length)
+  {
+    return unpack(to,from);
+  }
   virtual uint packed_col_length(const char *to, uint length)
   { return length;}
   virtual uint max_packed_col_length(uint max_length)
@@ -251,12 +255,13 @@ public:
 
 class Field_str :public Field {
 public:
+  double ceiling; // for ::store(double nr)
   Field_str(char *ptr_arg,uint32 len_arg, uchar *null_ptr_arg,
 	    uchar null_bit_arg, utype unireg_check_arg,
 	    const char *field_name_arg,
 	    struct st_table *table_arg)
     :Field(ptr_arg, len_arg, null_ptr_arg, null_bit_arg,
-	   unireg_check_arg, field_name_arg, table_arg)
+	   unireg_check_arg, field_name_arg, table_arg), ceiling(0.0)
     {}
   Item_result result_type () const { return STRING_RESULT; }
   uint decimals() const { return NOT_FIXED_DEC; }
@@ -541,6 +546,13 @@ public:
 
 
 class Field_timestamp :public Field_num {
+#if MYSQL_VERSION_ID < 40100
+  /*
+    We save the original field length here because field_length is
+    changed to a mock value in case when the 'new_mode' is in effect.
+  */
+  uint32 orig_field_length;
+#endif
 public:
   Field_timestamp(char *ptr_arg, uint32 len_arg,
 		  enum utype unireg_check_arg, const char *field_name_arg,
@@ -582,7 +594,11 @@ public:
   void fill_and_store(char *from,uint len);
   bool get_date(TIME *ltime,bool fuzzydate);
   bool get_time(TIME *ltime);
-  void make_field(Send_field *field);
+
+#if MYSQL_VERSION_ID < 40100
+  friend TABLE *open_table(THD *thd,const char *db,const char *table_name,
+                           const char *alias,bool *refresh);
+#endif
 };
 
 
@@ -690,6 +706,7 @@ public:
   double val_real(void);
   longlong val_int(void);
   String *val_str(String*,String *);
+  bool get_date(TIME *ltime,bool fuzzydate);
   bool get_time(TIME *ltime);
   int cmp(const char *,const char*);
   void sort_string(char *buff,uint length);
@@ -889,6 +906,7 @@ public:
   inline uint32 get_length(uint row_offset=0)
   { return get_length(ptr+row_offset); }
   uint32 get_length(const char *ptr);
+  void put_length(char *pos, uint32 length);
   bool binary() const { return binary_flag; }
   inline void get_ptr(char **str)
     {
@@ -922,6 +940,7 @@ public:
   const char *unpack(char *to, const char *from);
   char *pack_key(char *to, const char *from, uint max_length);
   char *pack_key_from_key_image(char* to, const char *from, uint max_length);
+  const char *unpack_key(char* to, const char *from, uint max_length);
   int pack_cmp(const char *a, const char *b, uint key_length);
   int pack_cmp(const char *b, uint key_length);
   uint packed_col_length(const char *col_ptr, uint length);
