@@ -240,6 +240,7 @@ public:
   void fix_length_and_dec();
   const char *func_name() const { return "nullif"; }
   table_map not_null_tables() const { return 0; }
+  bool is_null();
 };
 
 
@@ -469,6 +470,7 @@ public:
     if (!args[0]->maybe_null)
     {
       used_tables_cache= 0;			/* is always false */
+      const_item_cache= 1;
       cached_value= (longlong) 0;
     }
     else
@@ -583,7 +585,6 @@ public:
   Item_cond() : Item_bool_func(), abort_on_null(1) { const_item_cache=0; }
   Item_cond(Item *i1,Item *i2) :Item_bool_func(), abort_on_null(0)
   { list.push_back(i1); list.push_back(i2); }
-  ~Item_cond() { list.delete_elements(); }
   bool add(Item *item) { return list.push_back(item); }
   bool fix_fields(THD *,struct st_table_list *);
 
