@@ -295,7 +295,7 @@ class Item_dec_func :public Item_real_func
   Item_dec_func(Item *a,Item *b) :Item_real_func(a,b) {}
   void fix_length_and_dec()
   {
-    decimals=6; max_length=float_length(decimals);
+    decimals=NOT_FIXED_DEC; max_length=float_length(decimals);
     maybe_null=1;
   }
   inline double fix_result(double value)
@@ -600,7 +600,7 @@ public:
   {
     maybe_null=0; max_length=3;
     used_tables_cache|= item->used_tables();
-    not_null_tables_cache&= item->not_null_tables();
+    not_null_tables_cache= item->not_null_tables();
     const_item_cache&=  item->const_item();
     with_sum_func= with_sum_func || item->with_sum_func;
   }
@@ -642,23 +642,30 @@ public:
   unsigned int size_of() { return sizeof(*this);}  
 };
 
+/* Base class for all bit functions: '~', '|', '^', '&', '>>', '<<' */
 
-class Item_func_bit_or :public Item_int_func
+class Item_func_bit: public Item_int_func
 {
 public:
-  Item_func_bit_or(Item *a,Item *b) :Item_int_func(a,b) {}
-  longlong val_int();
-  const char *func_name() const { return "|"; }
-  void fix_length_and_dec() { unsigned_flag=1; }
+  Item_func_bit(Item *a, Item *b) :Item_int_func(a, b) {}
+  Item_func_bit(Item *a) :Item_int_func(a) {}
+  void fix_length_and_dec() { unsigned_flag= 1; }
 };
 
-class Item_func_bit_and :public Item_int_func
+class Item_func_bit_or :public Item_func_bit
 {
 public:
-  Item_func_bit_and(Item *a,Item *b) :Item_int_func(a,b) {}
+  Item_func_bit_or(Item *a, Item *b) :Item_func_bit(a, b) {}
+  longlong val_int();
+  const char *func_name() const { return "|"; }
+};
+
+class Item_func_bit_and :public Item_func_bit
+{
+public:
+  Item_func_bit_and(Item *a, Item *b) :Item_func_bit(a, b) {}
   longlong val_int();
   const char *func_name() const { return "&"; }
-  void fix_length_and_dec() { unsigned_flag=1; }
 };
 
 class Item_func_bit_count :public Item_int_func
@@ -670,30 +677,28 @@ public:
   void fix_length_and_dec() { max_length=2; }
 };
 
-class Item_func_shift_left :public Item_int_func
+class Item_func_shift_left :public Item_func_bit
 {
 public:
-  Item_func_shift_left(Item *a,Item *b) :Item_int_func(a,b) {}
+  Item_func_shift_left(Item *a, Item *b) :Item_func_bit(a, b) {}
   longlong val_int();
   const char *func_name() const { return "<<"; }
-  void fix_length_and_dec() { unsigned_flag=1; }
 };
 
-class Item_func_shift_right :public Item_int_func
+class Item_func_shift_right :public Item_func_bit
 {
 public:
-  Item_func_shift_right(Item *a,Item *b) :Item_int_func(a,b) {}
+  Item_func_shift_right(Item *a, Item *b) :Item_func_bit(a, b) {}
   longlong val_int();
   const char *func_name() const { return ">>"; }
 };
 
-class Item_func_bit_neg :public Item_int_func
+class Item_func_bit_neg :public Item_func_bit
 {
 public:
-  Item_func_bit_neg(Item *a) :Item_int_func(a) {}
+  Item_func_bit_neg(Item *a) :Item_func_bit(a) {}
   longlong val_int();
   const char *func_name() const { return "~"; }
-  void fix_length_and_dec() { unsigned_flag=1; }
 };
 
 class Item_func_set_last_insert_id :public Item_int_func
@@ -888,13 +893,16 @@ class Item_func_set_user_var :public Item_func
   enum Item_result cached_result_type;
   LEX_STRING name;
   user_var_entry *entry;
+  char buffer[MAX_FIELD_WIDTH];
+  String value;
 
 public:
-  Item_func_set_user_var(LEX_STRING a,Item *b): Item_func(b), name(a) {}
+  Item_func_set_user_var(LEX_STRING a,Item *b):
+    Item_func(b), name(a), value(buffer,sizeof(buffer)) {}
   double val();
   longlong val_int();
   String *val_str(String *str);
-  void update_hash(void *ptr, uint length, enum Item_result type);
+  bool update_hash(const void *ptr, uint length, enum Item_result type);
   bool update();
   enum Item_result result_type () const { return cached_result_type; }
   bool fix_fields(THD *thd,struct st_table_list *tables);
@@ -913,7 +921,6 @@ class Item_func_get_user_var :public Item_func
 public:
   Item_func_get_user_var(LEX_STRING a):
     Item_func(), name(a) {}
-  user_var_entry *get_entry();
   double val();
   longlong val_int();
   String *val_str(String* str);
@@ -1016,13 +1023,12 @@ enum Item_cast
 Item *create_func_cast(Item *a, Item_cast cast_type);
 
 
-class Item_func_bit_xor : public Item_int_func
+class Item_func_bit_xor : public Item_func_bit
 {
 public:
-  Item_func_bit_xor(Item *a,Item *b) :Item_int_func(a,b) {}
+  Item_func_bit_xor(Item *a, Item *b) :Item_func_bit(a, b) {}
   longlong val_int();
   const char *func_name() const { return "^"; }
-  void fix_length_xor_dec() { unsigned_flag=1; }
 };
 
 class Item_func_is_free_lock :public Item_int_func
