@@ -78,9 +78,9 @@ extern "C" void free_user_var(user_var_entry *entry)
 ** Thread specific functions
 ****************************************************************************/
 
-THD::THD():user_time(0),fatal_error(0),last_insert_id_used(0),
-	   insert_id_used(0),rand_used(0),in_lock_tables(0),
-	   global_read_lock(0),bootstrap(0)
+THD::THD():user_time(0),global_read_lock(0),fatal_error(0),
+           last_insert_id_used(0),insert_id_used(0),rand_used(0),
+           in_lock_tables(0),bootstrap(0)
 {
   host=user=priv_user=db=query=ip=0;
   host_or_ip= "connecting host";
@@ -90,6 +90,7 @@ THD::THD():user_time(0),fatal_error(0),last_insert_id_used(0),
   query_error=0;
   next_insert_id=last_insert_id=0;
   open_tables=temporary_tables=handler_tables=0;
+  hash_clear(&handler_tables_hash);
   current_tablenr=0;
   handler_items=0;
   tmp_table=0;
@@ -215,11 +216,9 @@ void THD::cleanup(void)
     lock=locked_tables; locked_tables=0;
     close_thread_tables(this);
   }
-  if (handler_tables)
-  {
-    open_tables=handler_tables; handler_tables=0;
-    close_thread_tables(this);
-  }
+  mysql_ha_flush(this, (TABLE_LIST*) 0,
+                 MYSQL_HA_CLOSE_FINAL | MYSQL_HA_FLUSH_ALL);
+  hash_free(&handler_tables_hash);
   close_temporary_tables(this);
   hash_free(&user_vars);
   if (global_read_lock)
@@ -299,8 +298,18 @@ void THD::awake(bool prepare_to_die)
       exits the cond in the time between read and broadcast, but that is
       ok since all we want to do is to make the victim thread get out
       of waiting on current_cond.
+      If we see a non-zero current_cond: it cannot be an old value (because
+      then exit_cond() should have run and it can't because we have mutex); so
+      it is the true value but maybe current_mutex is not yet non-zero (we're
+      in the middle of enter_cond() and there is a "memory order
+      inversion"). So we test the mutex too to not lock 0.
+      Note that there is a small chance we fail to kill. If victim has locked
+      current_mutex, and hasn't entered enter_cond(), then we don't know it's
+      going to wait on cond. Then victim goes into its cond "forever" (until
+      we issue a second KILL). True we have set its thd->killed but it may not
+      see it immediately and so may have time to reach the cond_wait().
     */
-    if (mysys_var->current_cond)
+    if (mysys_var->current_cond && mysys_var->current_mutex)
     {
       pthread_mutex_lock(mysys_var->current_mutex);
       pthread_cond_broadcast(mysys_var->current_cond);
@@ -555,7 +564,7 @@ select_export::prepare(List<Item> &list)
     return 1;
   }
   /* Create the file world readable */
-  if ((file=my_create(path, 0666, O_WRONLY, MYF(MY_WME))) < 0)
+  if ((file=my_create(path, 0666, O_WRONLY|O_EXCL, MYF(MY_WME))) < 0)
     return 1;
 #ifdef HAVE_FCHMOD
   (void) fchmod(file,0666);			// Because of umask()
@@ -794,7 +803,7 @@ select_dump::prepare(List<Item> &list __attribute__((unused)))
     return 1;
   }
   /* Create the file world readable */
-  if ((file=my_create(path, 0666, O_WRONLY, MYF(MY_WME))) < 0)
+  if ((file=my_create(path, 0666, O_WRONLY|O_EXCL, MYF(MY_WME))) < 0)
     return 1;
 #ifdef HAVE_FCHMOD
   (void) fchmod(file,0666);			// Because of umask()
